# -*- coding: utf-8 -*-

"""
    eve.io.mongo.validation
    ~~~~~~~~~~~~~~~~~~~~~~~

    This module implements the mongo Validator class, used to validate that
    objects incoming via POST/PATCH requests conform to the API domain.
    An extension of Cerberus Validator.

    :copyright: (c) 2017 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""
import copy
from bson import ObjectId
from bson.dbref import DBRef
from collections import Mapping
from flask import current_app as app
from werkzeug.datastructures import FileStorage
from cerberus import Validator

from eve.auth import auth_field_and_value
from eve.io.mongo.geo import Point, MultiPoint, LineString, Polygon, \
    MultiLineString, MultiPolygon, GeometryCollection
from eve.utils import config, str_type
from eve.versioning import get_data_version_relation_document


class Validator(Validator):
    """ A cerberus.Validator subclass adding the `unique` contraint to
    Cerberus standard validation.

    :param schema: the validation schema, to be composed according to Cerberus
                   documentation.
    :param resource: the resource name.

    .. versionchanged:: 0.6.1
       __init__ signature update for cerberus v0.8.1 compatibility.
       Disable 'transparent_schema_rules' by default in favor of explicit
       validators for rules unsupported by cerberus. This can be overridden
       globally or on a per-resource basis through a config option.

    .. versionchanged:: 0.5
       Support for _original_document
       Fix crash bug with Cerberus 0.7.1+ and keyschema rule. See Cerberus #48.

    .. versionchanged:: 0.0.6
       Support for 'allow_unknown' which allows to successfully validate
       unknown key/value pairs.

    .. versionchanged:: 0.0.4
       Support for 'transparent_schema_rules' introduced with Cerberus 0.0.3,
       which allows for insertion of 'default' values in POST requests.
    """
    def __init__(self, schema=None, resource=None, allow_unknown=False,
                 transparent_schema_rules=False):
        self.resource = resource
        self._id = None
        self._original_document = None

        if resource:
            transparent_schema_rules = \
                config.DOMAIN[resource]['transparent_schema_rules']
            allow_unknown = config.DOMAIN[resource]['allow_unknown']
        super(Validator, self).__init__(
            schema,
            transparent_schema_rules=transparent_schema_rules,
            allow_unknown=allow_unknown)

    def validate_update(self, document, _id, original_document=None):
        """ Validate method to be invoked when performing an update, not an
        insert.

        :param document: the document to be validated.
        :param _id: the unique id of the document.
        """
        self._id = _id
        self._original_document = original_document
        return super(Validator, self).validate_update(document)

    def validate_replace(self, document, _id, original_document=None):
        """ Validation method to be invoked when performing a document
        replacement. This differs from :func:`validation_update` since in this
        case we want to perform a full :func:`validate` (the new document is to
        be considered a new insertion and required fields needs validation).
        However, like with validate_update, we also want the current _id
        not to be checked when validating 'unique' values.

        .. versionadded:: 0.1.0
        """
        self._id = _id
        self._original_document = original_document
        return super(Validator, self).validate(document)

    def _validate_default(self, unique, field, value):
        """ Fake validate function to let cerberus accept "default"
            as keyword in the schema

        .. versionadded:: 0.6.2
        """
        pass

    def _validate_versioned(self, unique, field, value):
        """ Fake validate function to let cerberus accept "versioned"
            as keyword in the schema

        .. versionadded:: 0.6.2
        """
        pass

    def _validate_unique_to_user(self, unique, field, value):
        """ Validates that a value is unique to the active user. Active user is
        the user authenticated for current request. See #646.

        .. versionadded: 0.6
        """

        auth_field, auth_value = auth_field_and_value(self.resource)

        # if an auth value has been set for this request, then make sure it is
        # taken into account when checking for value uniqueness.
        query = {auth_field: auth_value} if auth_field else {}

        self._is_value_unique(unique, field, value, query)

    def _validate_unique(self, unique, field, value):
        """ Enables validation for `unique` schema attribute.

        :param unique: Boolean, wether the field value should be
                       unique or not.
        :param field: field name.
        :param value: field value.

        .. versionchanged:: 0.6
           Validates field value uniqueness against the whole datasource,
           independently of the request method. See #646.

        .. versionchanged:: 0.3
           Support for new 'self._error' signature introduced with Cerberus
           v0.5.

        .. versionchanged:: 0.2
           Handle the case in which ID_FIELD is not of ObjectId type.
        """
        self._is_value_unique(unique, field, value, {})

    def _is_value_unique(self, unique, field, value, query):
        """ Validates that a field value is unique.

        .. versionchanged:: 0.6.2
           Exclude soft deleted documents from uniqueness check. Closes #831.

        .. versionadded:: 0.6
        """
        if unique:
            query[field] = value

            resource_config = config.DOMAIN[self.resource]

            # exclude soft deleted documents if applicable
            if resource_config['soft_delete']:
                # be aware that, should a previously (soft) deleted document be
                # restored, and because we explicitly ignore soft deleted
                # documents while validating 'unique' fields, there is a chance
                # that a unique field value will end up being now duplicated
                # in two documents: the restored one, and the one which has
                # been stored with the same field value while the original
                # document was in 'deleted' state.

                # we make sure to also include documents which are missing the
                # DELETED field. This happens when soft deletes are enabled on
                # an a resource with existing documents.
                query[config.DELETED] = {'$ne': True}

            # exclude current document
            if self._id:
                id_field = resource_config['id_field']
                query[id_field] = {'$ne': self._id}

            # we perform the check on the native mongo driver (and not on
            # app.data.find_one()) because in this case we don't want the usual
            # (for eve) query injection to interfere with this validation. We
            # are still operating within eve's mongo namespace anyway.

            datasource, _, _, _ = app.data.datasource(self.resource)
            if app.data.driver.db[datasource].find_one(query):
                self._error(field, "value '%s' is not unique" % value)

    def _validate_data_relation(self, data_relation, field, value):
        """ Enables validation for `data_relation` field attribute. Makes sure
        'value' of 'field' adheres to the referential integrity rule specified
        by 'data_relation'.

        :param data_relation: a dict following keys:
            'resource': foreign resource name
            'field': foreign field name
            'version': True if this relation points to a specific version
            'type': the type for the reference field if 'version': True
        :param field: field name.
        :param value: field value.

        .. versionchanged:: 0.4
           Support for document versioning.

        .. versionchanged:: 0.3
           Support for new 'self._error' signature introduced with Cerberus
           v0.5.

        .. versionchanged:: 0.1.1
           'collection' key renamed to 'resource' (data_relation)

        .. versionadded: 0.0.5
        """
        if 'version' in data_relation and data_relation['version'] is True:
            value_field = data_relation['field']
            version_field = app.config['VERSION']

            # check value format
            if isinstance(value, dict) and value_field in value \
                    and version_field in value:
                resource_def = config.DOMAIN[data_relation['resource']]
                if resource_def['versioning'] is False:
                    self._error(
                        field, "can't save a version with"
                        " data_relation if '%s' isn't versioned" %
                        data_relation['resource'])
                else:
                    search = get_data_version_relation_document(
                        data_relation, value)

                    if not search:
                        self._error(
                            field, "value '%s' must exist in resource"
                            " '%s', field '%s' at version '%s'." % (
                                value[value_field], data_relation['resource'],
                                data_relation['field'], value[version_field]))
            else:
                self._error(
                    field, "versioned data_relation must be a dict"
                    " with fields '%s' and '%s'" %
                    (value_field, version_field))
        else:
            if not isinstance(value, list):
                value = [value]

            data_resource = data_relation['resource']
            for item in value:
                    query = {data_relation['field']: item.id
                             if isinstance(item, DBRef) else item}
                    if not app.data.find_one(data_resource, None, **query):
                        self._error(
                            field,
                            "value '%s' must exist in resource"
                            " '%s', field '%s'." %
                            (item.id if isinstance(item, DBRef) else item,
                             data_resource, data_relation['field']))

    def _validate_type_objectid(self, field, value):
        """ Enables validation for `objectid` data type.

        :param field: field name.
        :param value: field value.

        .. versionchanged:: 0.3
           Support for new 'self._error' signature introduced with Cerberus
           v0.5.

        .. versionchanged:: 0.1.1
           regex check replaced with proper type check.
        """
        if not isinstance(value, ObjectId):
            self._error(field, "value '%s' cannot be converted to a ObjectId"
                        % value)

    def _validate_type_dbref(self, field, value):
        """ Enables validation for `DBRef` data type.

        :param field: field name.
        :param value: field value.

        """
        if not isinstance(value, DBRef):
            self._error(field, "value '%s' cannot be converted to a DBRef"
                        % value)

    def _validate_readonly(self, read_only, field, value):
        """
        .. versionchanged:: 0.5
           Not taking default values in consideration anymore since they are
           now being resolved after validation (#353).
           Consider the original value if available (#479).

        .. versionadded:: 0.4
        """
        original_value = self._original_document.get(field) \
            if self._original_document else None
        if value != original_value:
            super(Validator, self)._validate_readonly(read_only, field, value)

    def _validate_dependencies(self, document, dependencies, field,
                               break_on_error=False):
        """ With PATCH method, the validator is only provided with the updated
        fields. If an updated field depends on another field in order to be
        edited and the other field was previously set, the validator doesn't
        see it and rejects the update. In order to avoid that we merge the
        proposed changes with the original document before validating
        dependencies.

        .. versionchanged:: 0.6.1
           Fix: dependencies on sub-document fields are now properly
           processed (#706).

        .. versionchanged:: 0.6
           Fix: Only evaluate dependencies that don't have valid default
           values.

        .. versionchanged:: 0.5.1
           Fix: dependencies with value checking seems broken #547.

        .. versionadded:: 0.5
           If a dependency has a default value, skip it as Cerberus does not
           have the notion of default values and would report a missing
           dependency (#353).
           Fix for #363 (see docstring).
        """
        if dependencies is None:
            return True

        if isinstance(dependencies, str_type):
            dependencies = [dependencies]

        defaults = {}
        for d in dependencies:
            root = d.split('.')[0]
            default = self.schema[root].get('default')
            if default and root not in document:
                defaults[root] = default

        if isinstance(dependencies, Mapping):
            # Only evaluate dependencies that don't have *valid* defaults
            for k, v in defaults.items():
                if v in dependencies[k]:
                    del(dependencies[k])
        else:
            # Only evaluate dependencies that don't have defaults values
            dependencies = [d for d in dependencies if d not in
                            defaults.keys()]

        dcopy = None
        if self._original_document:
            dcopy = copy.copy(document)
            dcopy.update(self._original_document)
        return super(Validator, self)._validate_dependencies(dcopy or document,
                                                             dependencies,
                                                             field,
                                                             break_on_error)

    def _validate_type_media(self, field, value):
        """ Enables validation for `media` data type.

        :param field: field name.
        :param value: field value.

        .. versionadded:: 0.3
        """
        if not isinstance(value, FileStorage):
            self._error(field, "file was expected, got '%s' instead." % value)

    def _validate_type_point(self, field, value):
        """ Enables validation for `point` data type.

        :param field: field name.
        :param value: field value.
        """
        try:
            Point(value)
        except TypeError as e:
            self._error(field, "Point not correct %s: %s" % (value, e))

    def _validate_type_linestring(self, field, value):
        """ Enables validation for `linestring` data type.

        :param field: field name.
        :param value: field value.
        """
        try:
            LineString(value)
        except TypeError:
            self._error(field, "LineString not correct %s " % value)

    def _validate_type_polygon(self, field, value):
        """ Enables validation for `polygon` data type.

        :param field: field name.
        :param value: field value.
        """
        try:
            Polygon(value)
        except TypeError:
            self._error(field, "LineString not correct %s " % value)

    def _validate_type_multipoint(self, field, value):
        """ Enables validation for `multipoint` data type.

        :param field: field name.
        :param value: field value.
        """
        try:
            MultiPoint(value)
        except TypeError:
            self._error(field, "MultiPoint not correct" % value)

    def _validate_type_multilinestring(self, field, value):
        """ Enables validation for `multilinestring`data type.

        :param field: field name.
        :param value: field value.
        """
        try:
            MultiLineString(value)
        except TypeError:
            self._error(field, "MultiLineString not  correct" % value)

    def _validate_type_multipolygon(self, field, value):
        """ Enables validation for `multipolygon` data type.

        :param field: field name.
        :param value: field value.
        """
        try:
            MultiPolygon(value)
        except TypeError:
            self._error(field, "MultiPolygon not  correct" % value)

    def _validate_type_geometrycollection(self, field, value):
        """ Enables validation for `geometrycollection`data type

        :param field: field name.
        :param value: field nvalue
        """
        try:
            GeometryCollection(value)
        except TypeError:
            self._error(field, "GeometryCollection not correct" % value)

    def _error(self, field, _error):
        """ Change the default behaviour so that, if VALIDATION_ERROR_AS_LIST
        is enabled, single validation errors are returned as a list. See #536.

        :param field: field name
        :param _error: field error(s)

        .. versionadded:: 0.6
        """
        super(Validator, self)._error(field, _error)
        if config.VALIDATION_ERROR_AS_LIST:
            err = self._errors[field]
            if not isinstance(err, list):
                self._errors[field] = [err]



import simplejson as json

from bson import ObjectId
from eve import ETAG
from eve import ISSUES
from eve import LAST_UPDATED
from eve import STATUS
from eve import STATUS_OK
from eve.methods.patch import patch_internal
from eve.tests import TestBase
from eve.tests.test_settings import MONGO_DBNAME
from eve.tests.utils import DummyEvent


class TestPatch(TestBase):

    def test_patch_to_resource_endpoint(self):
        _, status = self.patch(self.known_resource_url, data={})
        self.assert405(status)

    def test_readonly_resource(self):
        _, status = self.patch(self.readonly_id_url, data={})
        self.assert405(status)

    def test_unknown_id(self):
        _, status = self.patch(self.unknown_item_id_url,
                               data={"key1": 'value1'})
        self.assert404(status)

    def test_unknown_id_different_resource(self):
        # patching a 'user' with a valid 'contact' id will 404
        _, status = self.patch('%s/%s/' % (self.different_resource,
                                           self.item_id),
                               data={"key1": "value1"})
        self.assert404(status)

        # of course we can still patch a 'user'
        _, status = self.patch('%s/%s/' % (self.different_resource,
                                           self.user_id),
                               data={'key1': '{"username": "username1"}'},
                               headers=[('If-Match', self.user_etag)])
        self.assert200(status)

    def test_by_name(self):
        _, status = self.patch(self.item_name_url, data={'key1': 'value1'})
        self.assert405(status)

    def test_ifmatch_missing(self):
        res, status = self.patch(self.item_id_url, data={'key1': 'value1'})
        self.assert428(status)

    def test_ifmatch_missing_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False
        r, status = self.patch(self.item_id_url, data={'key1': 'value1'})
        self.assert200(status)
        self.assertTrue(ETAG in r)

    def test_ifmatch_disabled(self):
        self.app.config['IF_MATCH'] = False
        r, status = self.patch(self.item_id_url, data={'key1': 'value1'})
        self.assert200(status)
        self.assertTrue(ETAG not in r)

    def test_ifmatch_disabled_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False
        self.app.config['IF_MATCH'] = False
        r, status = self.patch(self.item_id_url, data={'key1': 'value1'})
        self.assert200(status)
        self.assertTrue(ETAG not in r)

    def test_ifmatch_bad_etag(self):
        _, status = self.patch(self.item_id_url,
                               data={'key1': 'value1'},
                               headers=[('If-Match', 'not-quite-right')])
        self.assert412(status)

    def test_ifmatch_bad_etag_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False
        _, status = self.patch(self.item_id_url,
                               data={'key1': 'value1'},
                               headers=[('If-Match', 'not-quite-right')])
        self.assert412(status)

    def test_unique_value(self):
        # TODO
        # for the time being we are happy with testing only Eve's custom
        # validation. We rely on Cerberus' own test suite for other validation
        # unit tests. This test also makes sure that response status is
        # syntactically correct in case of validation issues.
        # We should probably test every single case as well (seems overkill).
        r, status = self.patch(self.item_id_url,
                               data={"ref": "%s" % self.alt_ref},
                               headers=[('If-Match', self.item_etag)])
        self.assertValidationErrorStatus(status)
        self.assertValidationError(r, {'ref': "value '%s' is not unique" %
                                       self.alt_ref})

    def test_patch_string(self):
        field = "ref"
        test_value = "1234567890123456789012345"
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_patch_integer(self):
        field = "prog"
        test_value = 9999
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_patch_list_as_array(self):
        field = "role"
        test_value = ["vendor", "client"]
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertTrue(set(test_value).issubset(db_value))

    def test_patch_rows(self):
        field = "rows"
        test_value = [
            {'sku': 'AT1234', 'price': 99},
            {'sku': 'XF9876', 'price': 9999}
        ]
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)

        for test_item in test_value:
            self.assertTrue(test_item in db_value)

    def test_patch_list(self):
        field = "alist"
        test_value = ["a_string", 99]
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_patch_dict(self):
        field = "location"
        test_value = {'address': 'an address', 'city': 'a city'}
        changes = {field: test_value}
        original_city = []

        def keep_original_city(resource_name, updates, original):
            original_city.append(original['location']['city'])

        self.app.on_update += keep_original_city
        self.app.on_updated += keep_original_city
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)
        self.assertEqual(original_city[0], original_city[1])

    def test_patch_datetime(self):
        field = "born"
        test_value = "Tue, 06 Nov 2012 10:33:31 GMT"
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_patch_objectid(self):
        field = "tid"
        test_value = "4f71c129c88e2018d4000000"
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_patch_null_objectid(self):
        # verify that #341 is fixed.
        field = "tid"
        test_value = None
        changes = {field: test_value}
        r = self.perform_patch(changes)
        db_value = self.compare_patch_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_patch_defaults(self):
        field = "ref"
        test_value = "1234567890123456789012345"
        changes = {field: test_value}
        r = self.perform_patch(changes)
        self.assertRaises(KeyError, self.compare_patch_with_get, 'title', r)

    def test_patch_defaults_with_post_override(self):
        field = "ref"
        test_value = "1234567890123456789012345"
        r = self.perform_patch_with_post_override(field, test_value)
        self.assert200(r.status_code)
        self.assertRaises(KeyError, self.compare_patch_with_get, 'title',
                          json.loads(r.get_data()))

    def test_patch_multiple_fields(self):
        fields = ['ref', 'prog', 'role']
        test_values = ["9876543210987654321054321", 123, ["agent"]]
        changes = {"ref": test_values[0], "prog": test_values[1],
                   "role": test_values[2]}
        r = self.perform_patch(changes)
        db_values = self.compare_patch_with_get(fields, r)
        for i in range(len(db_values)):
            self.assertEqual(db_values[i], test_values[i])

    def test_patch_with_post_override(self):
        # a POST request with PATCH override turns into a PATCH request
        r = self.perform_patch_with_post_override('prog', 1)
        self.assert200(r.status_code)

    def test_patch_internal(self):
        # test that patch_internal is available and working properly.
        test_field = 'ref'
        test_value = "9876543210987654321098765"
        data = {test_field: test_value}
        with self.app.test_request_context(self.item_id_url):
            r, _, _, status = patch_internal(
                self.known_resource, data, concurrency_check=False,
                **{'_id': self.item_id})
        db_value = self.compare_patch_with_get(test_field, r)
        self.assertEqual(db_value, test_value)
        self.assert200(status)

    def test_patch_etag_header(self):
        # test that Etag is always included with response header. See #562.
        changes = {"ref": "1234567890123456789012345"}
        headers = [('Content-Type', 'application/json'),
                   ('If-Match', self.item_etag)]
        r = self.test_client.patch(self.item_id_url,
                                   data=json.dumps(changes),
                                   headers=headers)
        self.assertTrue('Etag' in r.headers)

        # test that ETag is compliant to RFC 7232-2.3 and #794 is fixed.
        etag = r.headers['ETag']

        self.assertTrue(etag[0] == '"')
        self.assertTrue(etag[-1] == '"')

    def test_patch_etag_header_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False
        changes = {'ref': '1234567890123456789012345'}
        headers = [('Content-Type', 'application/json'),
                   ('If-Match', self.item_etag)]
        r, status = self.patch(
            self.item_id_url,
            data=json.dumps(changes),
            headers=headers
        )

        self.assertTrue(ETAG in r)
        self.assertTrue(self.item_etag != r[ETAG])

    def test_patch_nested(self):
        changes = {'location.city': 'a nested city',
                   'location.address': 'a nested address'}
        r = self.perform_patch(changes)
        values = self.compare_patch_with_get('location', r)
        self.assertEqual(values['city'], 'a nested city')
        self.assertEqual(values['address'], 'a nested address')

    def perform_patch(self, changes):
        r, status = self.patch(self.item_id_url,
                               data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert200(status)
        self.assertPatchResponse(r, self.item_id)
        return r

    def perform_patch_with_post_override(self, field, value):
        headers = [('X-HTTP-Method-Override', 'PATCH'),
                   ('If-Match', self.item_etag),
                   ('Content-Type', 'application/json')]
        return self.test_client.post(self.item_id_url,
                                     data=json.dumps({field: value}),
                                     headers=headers)

    def compare_patch_with_get(self, fields, patch_response):
        raw_r = self.test_client.get(self.item_id_url)
        r, status = self.parse_response(raw_r)
        self.assert200(status)
        self.assertEqual(raw_r.headers.get('ETag').replace('"', ''),
                         patch_response[ETAG])
        if isinstance(fields, str):
            return r[fields]
        else:
            return [r[field] for field in fields]

    def test_patch_allow_unknown(self):
        changes = {"unknown": "unknown"}
        r, status = self.patch(self.item_id_url,
                               data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assertValidationErrorStatus(status)
        self.assertValidationError(r, {'unknown': 'unknown field'})
        self.app.config['DOMAIN'][self.known_resource]['allow_unknown'] = True
        r, status = self.patch(self.item_id_url,
                               data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert200(status)
        self.assertPatchResponse(r, self.item_id)

    def test_patch_x_www_form_urlencoded(self):
        field = "ref"
        test_value = "1234567890123456789012345"
        changes = {field: test_value}
        headers = [('If-Match', self.item_etag)]
        r, status = self.parse_response(self.test_client.patch(
            self.item_id_url, data=changes, headers=headers))
        self.assert200(status)
        self.assertTrue('OK' in r[STATUS])

    def test_patch_x_www_form_urlencoded_number_serialization(self):
        del(self.domain['contacts']['schema']['ref']['required'])
        field = 'anumber'
        test_value = 3.5
        changes = {field: test_value}
        headers = [('If-Match', self.item_etag)]
        r, status = self.parse_response(self.test_client.patch(
            self.item_id_url, data=changes, headers=headers))
        self.assert200(status)
        self.assertTrue('OK' in r[STATUS])

    def test_patch_referential_integrity(self):
        data = {"person": self.unknown_item_id}
        headers = [('If-Match', self.invoice_etag)]
        r, status = self.patch(self.invoice_id_url, data=data, headers=headers)
        self.assertValidationErrorStatus(status)
        expected = ("value '%s' must exist in resource '%s', field '%s'" %
                    (self.unknown_item_id, 'contacts',
                     self.domain['contacts']['id_field']))
        self.assertValidationError(r, {'person': expected})

        data = {"person": self.item_id}
        r, status = self.patch(self.invoice_id_url, data=data, headers=headers)
        self.assert200(status)
        self.assertPatchResponse(r, self.invoice_id)

    def test_patch_write_concern_success(self):
        # 0 and 1 are the only valid values for 'w' on our mongod instance (1
        # is the default)
        self.domain['contacts']['mongo_write_concern'] = {'w': 0}
        field = "ref"
        test_value = "X234567890123456789012345"
        changes = {field: test_value}
        _, status = self.patch(self.item_id_url,
                               data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert200(status)

    def test_patch_write_concern_fail(self):
        # should get a 500 since there's no replicaset on the mongod instance
        self.domain['contacts']['mongo_write_concern'] = {'w': 2}
        field = "ref"
        test_value = "X234567890123456789012345"
        changes = {field: test_value}
        _, status = self.patch(self.item_id_url,
                               data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert500(status)

    def test_patch_missing_standard_date_fields(self):
        """Documents created outside the API context could be lacking the
        LAST_UPDATED and/or DATE_CREATED fields.
        """
        # directly insert a document, without DATE_CREATED e LAST_UPDATED
        # values.
        contacts = self.random_contacts(1, False)
        ref = 'test_update_field'
        contacts[0]['ref'] = ref
        _db = self.connection[MONGO_DBNAME]
        _db.contacts.insert(contacts)

        # now retrieve same document via API and get its etag, which is
        # supposed to be computed on default DATE_CREATED and LAST_UPDATAED
        # values.
        response, status = self.get(self.known_resource, item=ref)
        etag = response[ETAG]
        _id = response['_id']

        # attempt a PATCH with the new etag.
        field = "ref"
        test_value = "X234567890123456789012345"
        changes = {field: test_value}
        _, status = self.patch('%s/%s' % (self.known_resource_url, _id),
                               data=changes, headers=[('If-Match', etag)])
        self.assert200(status)

    def test_patch_subresource(self):
        _db = self.connection[MONGO_DBNAME]

        # create random contact
        fake_contact = self.random_contacts(1)
        fake_contact_id = _db.contacts.insert(fake_contact)[0]

        # update first invoice to reference the new contact
        _db.invoices.update({'_id': ObjectId(self.invoice_id)},
                            {'$set': {'person': fake_contact_id}})

        # GET all invoices by new contact
        response, status = self.get('users/%s/invoices/%s' %
                                    (fake_contact_id, self.invoice_id))
        etag = response[ETAG]

        data = {"inv_number": "new_number"}
        headers = [('If-Match', etag)]
        response, status = self.patch('users/%s/invoices/%s' %
                                      (fake_contact_id, self.invoice_id),
                                      data=data, headers=headers)
        self.assert200(status)
        self.assertPatchResponse(response, self.invoice_id, 'peopleinvoices')

    def test_patch_bandwidth_saver(self):
        changes = {'ref': '1234567890123456789012345'}

        # bandwidth_saver is on by default
        self.assertTrue(self.app.config['BANDWIDTH_SAVER'])
        r = self.perform_patch(changes)
        self.assertFalse('ref' in r)
        db_value = self.compare_patch_with_get(self.app.config['ETAG'], r)
        self.assertEqual(db_value, r[self.app.config['ETAG']])
        self.item_etag = r[self.app.config['ETAG']]

        # test return all fields (bandwidth_saver off)
        self.app.config['BANDWIDTH_SAVER'] = False
        r = self.perform_patch(changes)
        self.assertTrue('ref' in r)
        db_value = self.compare_patch_with_get(self.app.config['ETAG'], r)
        self.assertEqual(db_value, r[self.app.config['ETAG']])

    def test_patch_readonly_field_with_previous_document(self):
        schema = self.domain['contacts']['schema']
        del(schema['ref']['required'])

        # disable read-only on the field so we can store a value which is
        # also different form its default value.
        schema['read_only_field']['readonly'] = False
        changes = {'read_only_field': 'value'}
        r = self.perform_patch(changes)

        # resume read-only status for the field
        self.domain['contacts']['schema']['read_only_field']['readonly'] = True

        # test that if the read-only field is included with the payload and its
        # value is equal to the one stored with the document, validation
        # succeeds (#479).
        etag = r['_etag']
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', etag)])
        self.assert200(status)
        self.assertPatchResponse(r, self.item_id)

        # test that if the read-only field is included with the payload and its
        # value is different from the stored document, validation fails.
        etag = r['_etag']
        changes = {'read_only_field': 'another value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', etag)])
        self.assert422(status)
        self.assertTrue('is read-only' in r['_issues']['read_only_field'])

    def test_patch_nested_document_not_overwritten(self):
        """ Test that nested documents are not overwritten on PATCH and #519
        is fixed.
        """

        schema = {
            'sensor': {
                "type": "dict",
                "schema": {
                    "name": {"type": "string"},
                    "lon": {"type": "float"},
                    "lat": {"type": "float"},
                    "value": {"type": "float", "default": 10.3},
                    "dict": {
                        'type': 'dict',
                        'schema': {
                            'string': {'type': 'string'},
                            'int': {'type': 'integer'},
                        }
                    }
                }
            },
            'test': {
                'type': 'string',
                'readonly': True,
                'default': 'default'
            }
        }

        self.app.config['BANDWIDTH_SAVER'] = False
        self.app.register_resource('sensors', {'schema': schema})

        changes = {
            'sensor': {
                'name': 'device_name',
                'lon': 43.4,
                'lat': 1.31,
                'dict': {'int': 99}
            }
        }
        r, status = self.post("sensors", data=changes)
        self.assert201(status)

        id, etag, value, test, int = (
            r[self.domain['sensors']['id_field']],
            r[ETAG],
            r['sensor']['value'],
            r['test'],
            r['sensor']['dict']['int']
        )

        changes = {
            'sensor': {
                'lon': 10.0,
                'dict': {'string': 'hi'}
            }
        }

        r, status = self.patch(
            "/%s/%s" % ('sensors', id),
            data=changes,
            headers=[('If-Match', etag)]
        )
        self.assert200(status)

        etag, value, int = (
            r[ETAG],
            r['sensor']['value'],
            r['sensor']['dict']['int']
        )
        self.assertEqual(value, 10.3)
        self.assertEqual(test, 'default')
        self.assertEqual(int, 99)

    def test_patch_nested_document_nullable_missing(self):
        schema = {
            'sensor': {
                'type': 'dict',
                'schema': {
                    'name': {'type': 'string'},
                },
                'default': None,
            },
            'other': {
                'type': 'dict',
                'schema': {
                    'name': {'type': 'string'},
                },
            }
        }
        self.app.config['BANDWIDTH_SAVER'] = False
        self.app.register_resource('sensors', {'schema': schema})

        changes = {}

        r, status = self.post("sensors", data=changes)
        self.assert201(status)
        id, etag = r[self.domain['sensors']['id_field']], r[ETAG]
        self.assertTrue('sensor' in r)
        self.assertEqual(r['sensor'], None)
        self.assertFalse('other' in r)

        changes = {
            'sensor': {'name': 'device_name'},
            'other': {'name': 'other_name'},
        }

        r, status = self.patch(
            "/%s/%s" % ('sensors', id),
            data=changes,
            headers=[('If-Match', etag)]
        )
        self.assert200(status)
        self.assertEqual(r['sensor'], {'name': 'device_name'})
        self.assertEqual(r['other'], {'name': 'other_name'})

    def test_patch_dependent_field_on_origin_document(self):
        """ Test that when patching a field which is dependent on another and
        this other field is not provided with the patch but is still present
        on the target document, the patch will be accepted. See #363.
        """
        # this will fail as dependent field is missing even in the
        # document we are trying to update.
        del(self.domain['contacts']['schema']['dependency_field1']['default'])
        del(self.domain['contacts']['defaults']['dependency_field1'])
        changes = {'dependency_field2': 'value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert422(status)

        # update the stored document by adding dependency field.
        changes = {'dependency_field1': 'value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert200(status)

        # now the field2 update will be accepted as the dependency field is
        # present in the stored document already.
        etag = r['_etag']
        changes = {'dependency_field2': 'value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', etag)])
        self.assert200(status)

    def test_patch_dependent_field_value_on_origin_document(self):
        """ Test that when patching a field which is dependent on another and
        this other field is not provided with the patch but is still present
        on the target document, the patch will be accepted. See #363.
        """
        # this will fail as dependent field is missing even in the
        # document we are trying to update.
        changes = {'dependency_field3': 'value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert422(status)

        # update the stored document by setting the dependency field to
        # the required value.
        changes = {'dependency_field1': 'value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert200(status)

        # now the field2 update will be accepted as the dependency field is
        # present in the stored document already.
        etag = r['_etag']
        changes = {'dependency_field3': 'value'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', etag)])
        self.assert200(status)

    def test_id_field_in_document_fails(self):
        # since v0.6 we also allow the id field to be included with the POSTed
        # document, but not with PATCH since it is immutable
        self.app.config['IF_MATCH'] = False
        id_field = self.domain[self.known_resource]['id_field']
        data = {id_field: '55b2340538345bd048100ffe'}
        r, status = self.patch(self.item_id_url, data=data)
        self.assert400(status)
        self.assertTrue('immutable' in r['_error']['message'])

    def test_patch_custom_idfield(self):
        response, status = self.get('products?max_results=1')
        product = response['_items'][0]
        headers = [('If-Match', product[ETAG])]
        data = {'title': 'Awesome product'}
        r, status = self.patch('products/%s' % product['sku'], data=data,
                               headers=headers)
        self.assert200(status)

    def test_patch_type_coercion(self):
        schema = self.domain[self.known_resource]['schema']
        schema['aninteger']['coerce'] = lambda string: int(float(string))
        changes = {'ref': '1234567890123456789054321', 'aninteger': '42.3'}
        r, status = self.patch(self.item_id_url, data=changes,
                               headers=[('If-Match', self.item_etag)])
        self.assert200(status)
        r, status = self.get(r['_links']['self']['href'])
        self.assertEqual(r['aninteger'], 42)

    def assertPatchResponse(self, response, item_id, resource=None):
        id_field = self.domain[resource or self.known_resource]['id_field']
        self.assertTrue(STATUS in response)
        self.assertTrue(STATUS_OK in response[STATUS])
        self.assertFalse(ISSUES in response)
        self.assertTrue(id_field in response)
        self.assertEqual(response[id_field], item_id)
        self.assertTrue(LAST_UPDATED in response)
        self.assertTrue(ETAG in response)
        self.assertTrue('_links' in response)
        self.assertItemLink(response['_links'], item_id)

    def patch(self, url, data, headers=[]):
        headers.append(('Content-Type', 'application/json'))
        r = self.test_client.patch(url,
                                   data=json.dumps(data),
                                   headers=headers)
        return self.parse_response(r)


class TestEvents(TestBase):
    new_ref = "0123456789012345678901234"

    def test_on_pre_PATCH(self):
        devent = DummyEvent(self.before_update)
        self.app.on_pre_PATCH += devent
        self.patch()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(3, len(devent.called))

    def test_on_pre_PATCH_contacts(self):
        devent = DummyEvent(self.before_update)
        self.app.on_pre_PATCH_contacts += devent
        self.patch()
        self.assertEqual(2, len(devent.called))

    def test_on_PATCH_dynamic_filter(self):
        def filter_this(resource, request, lookup):
            lookup["_id"] = self.unknown_item_id
        self.app.on_pre_PATCH += filter_this
        # Would normally patch the known document; will return 404 instead.
        r, s = self.parse_response(self.patch())
        self.assert404(s)

    def test_on_post_PATCH(self):
        devent = DummyEvent(self.after_update)
        self.app.on_post_PATCH += devent
        self.patch()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(200, devent.called[2].status_code)
        self.assertEqual(3, len(devent.called))

    def test_on_post_PATCH_contacts(self):
        devent = DummyEvent(self.after_update)
        self.app.on_post_PATCH_contacts += devent
        self.patch()
        self.assertEqual(200, devent.called[1].status_code)
        self.assertEqual(2, len(devent.called))

    def test_on_update(self):
        devent = DummyEvent(self.before_update)
        self.app.on_update += devent
        self.patch()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(3, len(devent.called))

    def test_on_update_contacts(self):
        devent = DummyEvent(self.before_update)
        self.app.on_update_contacts += devent
        self.patch()
        self.assertEqual(2, len(devent.called))

    def test_on_updated(self):
        devent = DummyEvent(self.after_update)
        self.app.on_updated += devent
        self.patch()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(3, len(devent.called))

    def test_on_updated_contacts(self):
        devent = DummyEvent(self.after_update)
        self.app.on_updated_contacts += devent
        self.patch()
        self.assertEqual(2, len(devent.called))

    def before_update(self):
        db = self.connection[MONGO_DBNAME]
        contact = db.contacts.find_one(ObjectId(self.item_id))
        return contact['ref'] == self.item_name

    def after_update(self):
        return not self.before_update()

    def patch(self):
        headers = [('Content-Type', 'application/json'),
                   ('If-Match', self.item_etag)]
        data = json.dumps({"ref": self.new_ref})
        return self.test_client.patch(
            self.item_id_url, data=data, headers=headers)



import unittest

from eve.defaults import build_defaults, resolve_default_values


class TestBuildDefaults(unittest.TestCase):
    def test_schemaless_dict(self):
        schema = {
            "address": {
                'type': 'dict'
            }
        }
        self.assertEqual({}, build_defaults(schema))

    def test_simple(self):
        schema = {
            "name": {'type': 'string'},
            "email": {'type': 'string', 'default': "no@example.com"}
        }
        res = build_defaults(schema)
        self.assertEqual({'email': 'no@example.com'}, res)

    def test_nested_one_level(self):
        schema = {
            "address": {
                'type': 'dict',
                'schema': {
                    'street': {'type': 'string'},
                    'country': {'type': 'string', 'default': 'wonderland'}
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({'address': {'country': 'wonderland'}}, res)

    def test_empty_defaults_multiple_level(self):
        schema = {
            'subscription': {
                'type': 'dict',
                'schema': {
                    'type': {'type': 'string'},
                    'when': {
                        'type': 'dict',
                        'schema': {
                            'timestamp': {'type': 'int'},
                            'repr': {'type': 'string'}
                        }
                    }
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({}, res)

    def test_nested_multilevel(self):
        schema = {
            "subscription": {
                'type': 'dict',
                'schema': {
                    'type': {'type': 'string'},
                    'when': {
                        'type': 'dict',
                        'schema': {
                            'timestamp': {'type': 'int', 'default': 0},
                            'repr': {'type': 'string', 'default': '0'}
                        }
                    }
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual(
            {'subscription': {'when': {'timestamp': 0, 'repr': '0'}}},
            res)

    def test_default_in_list_schema(self):
        schema = {
            "one": {
                'type': 'list',
                'schema': {
                    'type': 'dict',
                    'schema': {
                        'title': {
                            'type': 'string',
                            'default': 'M.'
                        }
                    }
                }
            },
            "two": {
                'type': 'list',
                'schema': {
                    'type': 'dict',
                    'schema': {
                        'name': {'type': 'string'}
                    }
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({"one": [{'title': 'M.'}]}, res)

    def test_default_in_list_without_schema(self):
        schema = {
            "one": {
                'type': 'list',
                'schema': {
                    'type': 'string',
                    'default': 'item'
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({"one": ['item']}, res)

    def test_lists_of_lists_with_default(self):
        schema = {
            'twisting': {
                'type': 'list',  # list of groups
                'required': True,
                'schema': {
                    'type': 'list',  # list of signals (in one group)
                    'schema': {
                        'type': 'string',
                        'default': 'listoflist',
                    }
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({'twisting': [['listoflist']]}, res)

    def test_lists_of_lists_without_default(self):
        schema = {
            'twisting': {
                'type': 'list',  # list of groups
                'required': True,
                'schema': {
                    'type': 'list',  # list of signals (in one group)
                    'schema': {
                        'type': 'ObjectId',
                        'required': True
                    }
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({}, res)

    def test_lists_of_lists_with_a_dict(self):
        schema = {
            'twisting': {
                'type': 'list',  # list of groups
                'required': True,
                'schema': {
                    'type': 'list',  # list of signals (in one group)
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            'name': {
                                'type': 'string',
                                'default': 'me'
                            }
                        },
                    }
                }
            }
        }
        res = build_defaults(schema)
        self.assertEqual({'twisting': [[{'name': 'me'}]]}, res)


class TestResolveDefaultValues(unittest.TestCase):
    def test_one_level(self):
        document = {'name': 'john'}
        defaults = {'email': 'noemail'}
        resolve_default_values(document, defaults)
        self.assertEqual({'name': 'john', 'email': 'noemail'}, document)

    def test_multilevel(self):
        document = {'name': 'myname', 'one': {'hey': 'jude'}}
        defaults = {'one': {'two': {'three': 'banana'}}}
        resolve_default_values(document, defaults)
        expected = {
            'name': 'myname',
            'one': {
                'hey': 'jude',
                'two': {'three': 'banana'}
            }
        }
        self.assertEqual(expected, document)

    def test_value_instead_of_dict(self):
        document = {'name': 'john'}
        defaults = {'name': {'first': 'john'}}
        resolve_default_values(document, defaults)
        self.assertEqual(document, defaults)

    def test_lists(self):
        document = {"one": [{"name": "john"}, {}]}
        defaults = {"one": [{"title": "M."}]}
        resolve_default_values(document, defaults)
        expected = {"one": [
            {"name": "john", "title": "M."},
            {"title": "M."}]}
        self.assertEqual(expected, document)

    def test_list_of_list_single_value(self):
        document = {'one': [[], []]}
        defaults = {'one': [['listoflist']]}
        resolve_default_values(document, defaults)
        # This functionality is not supported, no change in the document
        expected = {'one': [[], []]}
        assert expected == document

    def test_list_empty_list_as_default(self):
        # test that a default value of [] for a list does not causes IndexError
        # (#417).
        document = {'a': ['b']}
        defaults = {'a': []}
        resolve_default_values(document, defaults)
        expected = {'a': ['b']}
        assert expected == document

    def test_list_of_strings_as_default(self):
        document = {}
        defaults = {'a': ['b']}
        resolve_default_values(document, defaults)
        expected = {'a': ['b']}
        assert expected == document
        # overwrite defaults
        document = {'a': ['c', 'd']}
        defaults = {'a': ['b']}
        resolve_default_values(document, defaults)
        expected = {'a': ['c', 'd']}
        assert expected == document

    def test_list_of_list_dict_value(self):
        document = {'one': [[{}], [{}]]}
        defaults = {'one': [[{'name': 'banana'}]]}
        resolve_default_values(document, defaults)
        expected = {'one': [[{'name': 'banana'}], [{'name': 'banana'}]]}
        assert expected == document



# -*- coding: utf-8 -*-

"""
    Eve
    ~~~

    An out-of-the-box REST Web API that's as dangerous as you want it to be.

    :copyright: (c) 2017 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.

    .. versionchanged:: 0.5
       'SERVER_NAME' removed.
       'QUERY_WHERE' added.
       'QUERY_SORT' added.
       'QUERY_PAGE' added.
       'QUERY_MAX_RESULTS' added.
       'QUERY_PROJECTION' added.
       'QUERY_EMBEDDED' added.
       'RFC1123_DATE_FORMAT' added.

    .. versionchanged:: 0.4
       'META' defaults to '_meta'.
       'ERROR' defaults to '_error'.
       Remove unnecessary commented code.

    .. versionchanged:: 0.2
       'LINKS' defaults to '_links'.
       'ITEMS' defaults to '_items'.
       'STATUS' defaults to 'status'.
       'ISSUES' defaults to 'issues'.

    .. versionchanged:: 0.1.1
       'SERVER_NAME' defaults to None.

    .. versionchagned:: 0.0.9
       'DATE_FORMAT now using GMT instead of UTC.

"""

__version__ = '0.7.8'

# RFC 1123 (ex RFC 822)
DATE_FORMAT = '%a, %d %b %Y %H:%M:%S GMT'
RFC1123_DATE_FORMAT = '%a, %d %b %Y %H:%M:%S GMT'

URL_PREFIX = ''
API_VERSION = ''
PAGINATION = True
PAGINATION_LIMIT = 50
PAGINATION_DEFAULT = 25
ID_FIELD = '_id'
CACHE_CONTROL = 'max-age=10,must-revalidate'        # TODO confirm this value
CACHE_EXPIRES = 10

RESOURCE_METHODS = ['GET']
ITEM_METHODS = ['GET']
ITEM_LOOKUP = True
ITEM_LOOKUP_FIELD = ID_FIELD
ITEM_URL = 'regex("[a-f0-9]{24}")'

STATUS_OK = "OK"
STATUS_ERR = "ERR"
LAST_UPDATED = '_updated'
DATE_CREATED = '_created'
ISSUES = '_issues'
STATUS = '_status'
ERROR = '_error'
ITEMS = '_items'
LINKS = '_links'
ETAG = '_etag'
VERSION = '_version'
META = '_meta'
INFO = None

QUERY_WHERE = 'where'
QUERY_SORT = 'sort'
QUERY_PAGE = 'page'
QUERY_MAX_RESULTS = 'max_results'
QUERY_EMBEDDED = 'embedded'
QUERY_PROJECTION = 'projection'

VALIDATION_ERROR_STATUS = 422
VALIDATION_ERROR_AS_LIST = False

# must be the last line (will raise W402 on pyflakes)
from eve.flaskapp import Eve  # noqa



import time
from datetime import datetime

import simplejson as json
from bson import ObjectId
from bson.dbref import DBRef

from eve.methods.common import serialize, normalize_dotted_fields
from eve.tests import TestBase
from eve.tests.auth import ValidBasicAuth, ValidTokenAuth, ValidHMACAuth
from eve.tests.test_settings import MONGO_DBNAME
from eve.utils import config


class TestSerializer(TestBase):
    def test_serialize_subdocument(self):
        # tests fix for #244, serialization of sub-documents.
        schema = {'personal': {'type': 'dict',
                               'schema': {'best_friend': {'type': 'objectid'},
                                          'born': {'type': 'datetime'}}},
                  'without_type': {}}
        doc = {'personal': {'best_friend': '50656e4538345b39dd0414f0',
                            'born': 'Tue, 06 Nov 2012 10:33:31 GMT'},
               'without_type': 'foo'}
        with self.app.app_context():
            serialized = serialize(doc, schema=schema)
        self.assertTrue(
            isinstance(serialized['personal']['best_friend'], ObjectId))
        self.assertTrue(
            isinstance(serialized['personal']['born'], datetime))

    def test_mongo_serializes(self):
        schema = {
            'id': {'type': 'objectid'},
            'date': {'type': 'datetime'},
            'count': {'type': 'integer'},
            'average': {'type': 'float'},
            'dict_valueschema': {
                'valueschema': {'type': 'objectid'}
            },
            'refobj': {'type': 'dbref'}
        }
        with self.app.app_context():
            # Success
            res = serialize(
                {
                    'id': '50656e4538345b39dd0414f0',
                    'date': 'Tue, 06 Nov 2012 10:33:31 GMT',
                    'count': 42,
                    'average': 42.42,
                    'dict_valueschema': {
                        'foo1': '50656e4538345b39dd0414f0',
                        'foo2': '50656e4538345b39dd0414f0',
                    },
                    'refobj': {
                        '$id': '50656e4538345b39dd0414f0',
                        '$col': 'SomeCollection'
                    }
                },
                schema=schema
            )
            self.assertTrue(isinstance(res['id'], ObjectId))
            self.assertTrue(isinstance(res['date'], datetime))
            self.assertTrue(isinstance(res['count'], int))
            self.assertTrue(isinstance(res['average'], float))

            ks = res['dict_valueschema']
            self.assertTrue(isinstance(ks['foo1'], ObjectId))
            self.assertTrue(isinstance(ks['foo2'], ObjectId))
            self.assertTrue(isinstance(res['refobj'], DBRef))

    def test_non_blocking_on_simple_field_serialization_exception(self):
        schema = {
            'extract_time': {'type': 'datetime'},
            'date': {'type': 'datetime'},
            'total': {'type': 'integer'}
        }

        with self.app.app_context():
            # Success
            res = serialize(
                {
                    'extract_time': 'Tue, 06 Nov 2012 10:33:31 GMT',
                    'date': 'Tue, 06 Nov 2012 10:33:31 GMT',
                    'total': 'r123'
                },
                schema=schema
            )
            # this has been left untouched as it could not be serialized.
            self.assertEqual(res['total'], 'r123')
            # these have been both serialized.
            self.assertTrue(isinstance(res['extract_time'], datetime))
            self.assertTrue(isinstance(res['date'], datetime))

    def test_serialize_lists_of_lists(self):
        # serialize should handle list of lists of basic types
        schema = {
            'l_of_l': {
                'type': 'list',
                'schema': {
                    'type': 'list',
                    'schema': {
                        'type': 'objectid'
                    }
                }
            }
        }
        doc = {
            'l_of_l': [
                ['50656e4538345b39dd0414f0', '50656e4538345b39dd0414f0'],
                ['50656e4538345b39dd0414f0', '50656e4538345b39dd0414f0']
            ]
        }

        with self.app.app_context():
            serialized = serialize(doc, schema=schema)
        for sublist in serialized['l_of_l']:
            for item in sublist:
                self.assertTrue(isinstance(item, ObjectId))

        # serialize should handle list of lists of dicts
        schema = {
            'l_of_l': {
                'type': 'list',
                'schema': {
                    'type': 'list',
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            '_id': {
                                'type': 'objectid'
                            }
                        }
                    }
                }
            }
        }
        doc = {
            'l_of_l': [
                [
                    {'_id': '50656e4538345b39dd0414f0'},
                    {'_id': '50656e4538345b39dd0414f0'}
                ],
                [
                    {'_id': '50656e4538345b39dd0414f0'},
                    {'_id': '50656e4538345b39dd0414f0'}
                ],
            ]
        }
        with self.app.app_context():
            serialized = serialize(doc, schema=schema)
        for sublist in serialized['l_of_l']:
            for item in sublist:
                self.assertTrue(isinstance(item['_id'], ObjectId))

    def test_dbref_serialize_lists_of_lists(self):
        # serialize should handle list of lists of basic types
        schema = {
            'l_of_l': {
                'type': 'list',
                'schema': {
                    'type': 'list',
                    'schema': {
                        'type': 'dbref'
                    }
                }
            }
        }
        doc = {
            'l_of_l': [
                [{'$col': 'SomeCollection', '$id': '50656e4538345b39dd0414f0'},
                 {'$col': 'SomeCollection', '$id': '50656e4538345b39dd0414f0'}
                 ],
                [{'$col': 'SomeCollection', '$id': '50656e4538345b39dd0414f0'},
                 {'$col': 'SomeCollection', '$id': '50656e4538345b39dd0414f0'}
                 ]
            ]
        }

        with self.app.app_context():
            serialized = serialize(doc, schema=schema)
        for sublist in serialized['l_of_l']:
            for item in sublist:
                self.assertTrue(isinstance(item, DBRef))

        # serialize should handle list of lists of dicts
        schema = {
            'l_of_l': {
                'type': 'list',
                'schema': {
                    'type': 'list',
                    'schema': {
                        'type': 'dict',
                        'schema': {
                            '_id': {
                                'type': 'dbref'
                            }
                        }
                    }
                }
            }
        }
        doc = {
            'l_of_l': [
                [
                    {'_id': {'$col': 'SomeCollection',
                             '$id': '50656e4538345b39dd0414f0'}
                     },
                    {'_id': {'$col': 'SomeCollection',
                             '$id': '50656e4538345b39dd0414f0'}
                     }
                ],
                [
                    {'_id': {'$col': 'SomeCollection',
                             '$id': '50656e4538345b39dd0414f0'}
                     },
                    {'_id': {'$col': 'SomeCollection',
                             '$id': '50656e4538345b39dd0414f0'}
                     }
                ],
            ]
        }
        with self.app.app_context():
            serialized = serialize(doc, schema=schema)
        for sublist in serialized['l_of_l']:
            for item in sublist:
                self.assertTrue(isinstance(item['_id'], DBRef))

    def test_serialize_null_dictionary(self):
        # Serialization should continue after encountering a null value dict
        # field. Field may be nullable, or error will be caught in validation.
        schema = {
            'nullable_dict': {
                'type': 'dict',
                'nullable': True,
                'schema': {
                    'simple_field': {
                        'type': 'number'
                    }
                }
            }
        }
        doc = {
            'nullable_dict': None
        }
        with self.app.app_context():
            try:
                serialize(doc, schema=schema)
            except Exception:
                self.assertTrue(False, "Serializing null dictionaries should "
                                       "not raise an exception.")

    def test_serialize_null_list(self):
        schema = {
            'nullable_list': {
                'type': 'list',
                'nullable': True,
                'schema': {
                    'type': 'objectid'
                }
            }
        }
        doc = {
            'nullable_list': None
        }
        with self.app.app_context():
            try:
                serialize(doc, schema=schema)
            except Exception:
                self.fail('Serializing null lists'
                          ' should not raise an exception')

        schema = {
            'nullable_list': {
                'type': 'list',
                'nullable': True,
                'schema': {
                    'type': 'dbref'
                }
            }
        }
        doc = {
            'nullable_list': None
        }
        with self.app.app_context():
            try:
                serialize(doc, schema=schema)
            except Exception:
                self.fail('Serializing null lists'
                          ' should not raise an exception')

    def test_serialize_number(self):
        schema = {
            'anumber': {
                'type': 'number',
            }
        }
        for expected_type, value in [(int, '35'), (float, '3.5')]:
            doc = {
                'anumber': value
            }
            with self.app.app_context():
                serialized = serialize(doc, schema=schema)
                self.assertTrue(
                    isinstance(serialized['anumber'], expected_type)
                )

    def test_serialize_boolean(self):
        schema = {'bool': {'type': 'boolean'}}

        with self.app.app_context():
            for val in [1, '1', 0, '0', 'true', 'True', 'false', 'False']:
                doc = {'bool': val}
                serialized = serialize(doc, schema=schema)
                self.assertTrue(isinstance(serialized['bool'], bool))

    def test_serialize_inside_x_of_rules(self):
        for x_of in ['allof', 'anyof', 'oneof', 'noneof']:
            schema = {
                'x_of-field': {
                    x_of: [
                        {'type': 'objectid'},
                        {'required': True}
                    ]
                }
            }
            doc = {'x_of-field': '50656e4538345b39dd0414f0'}
            with self.app.app_context():
                serialized = serialize(doc, schema=schema)
                self.assertTrue(isinstance(serialized['x_of-field'], ObjectId))

    def test_serialize_inside_nested_x_of_rules(self):
        schema = {
            'nested-x_of-field': {
                'oneof': [
                    {
                        'anyof': [
                            {'type': 'objectid'},
                            {'type': 'datetime'}
                        ],
                        'required': True
                    },
                    {
                        'allof': [
                            {'type': 'boolean'},
                            {'required': True}
                        ]
                    }
                ]
            }
        }
        doc = {'nested-x_of-field': '50656e4538345b39dd0414f0'}
        with self.app.app_context():
            serialized = serialize(doc, schema=schema)
            self.assertTrue(
                isinstance(serialized['nested-x_of-field'], ObjectId))

    def test_serialize_inside_x_of_typesavers(self):
        for x_of in ['allof', 'anyof', 'oneof', 'noneof']:
            schema = {
                'x_of-field': {
                    '{0}_type'.format(x_of): ['objectid', 'float', 'boolean']
                }
            }
            doc = {'x_of-field': '50656e4538345b39dd0414f0'}
            with self.app.app_context():
                serialized = serialize(doc, schema=schema)
                self.assertTrue(isinstance(serialized['x_of-field'], ObjectId))

    def test_serialize_inside_list_of_x_of_rules(self):
        for x_of in ['allof', 'anyof', 'oneof', 'noneof']:
            schema = {
                'list-field': {
                    'type': 'list',
                    'schema': {
                        x_of: [
                            {
                                'type': 'objectid',
                                'required': True}
                        ]
                    }
                }
            }
            doc = {'list-field': ['50656e4538345b39dd0414f0']}
            with self.app.app_context():
                serialized = serialize(doc, schema=schema)
                serialized_oid = serialized['list-field'][0]
                self.assertTrue(isinstance(serialized_oid, ObjectId))

    def test_serialize_inside_list_of_schema_of_x_of_rules(self):
        for x_of in ['allof', 'anyof', 'oneof', 'noneof']:
            schema = {
                'list-field': {
                    'type': 'list',
                    'schema': {
                        x_of: [
                            {
                                'type': 'dict',
                                'schema': {
                                    'x_of-field': {
                                        'type': 'objectid',
                                        'required': True
                                    }
                                }
                            }
                        ]
                    }
                }
            }
            doc = {'list-field': [{'x_of-field': '50656e4538345b39dd0414f0'}]}
            with self.app.app_context():
                serialized = serialize(doc, schema=schema)
                serialized_oid = serialized['list-field'][0]['x_of-field']
                self.assertTrue(isinstance(serialized_oid, ObjectId))

    def test_serialize_inside_list_of_x_of_typesavers(self):
        for x_of in ['allof', 'anyof', 'oneof', 'noneof']:
            schema = {
                'list-field': {
                    'type': 'list',
                    'schema': {
                        '{0}_type'.format(x_of): [
                            'objectid', 'float', 'boolean'
                        ]
                    }
                }
            }
            doc = {'list-field': ['50656e4538345b39dd0414f0']}
            with self.app.app_context():
                serialized = serialize(doc, schema=schema)
                serialized_oid = serialized['list-field'][0]
                self.assertTrue(isinstance(serialized_oid, ObjectId))


class TestNormalizeDottedFields(TestBase):
    def test_normalize_dotted_fields(self):
        def compare_recursive(a, b):
            for key, value in a.items():
                if key not in b:
                    return False
                if isinstance(value, dict):
                    compare_recursive(value, b[key])
            return True

        document = {
            'a.b': 1,
            'c.d': {
                'e.f': {
                    'g': 1,
                    'h': 2,
                },
                'e.f.i': {'j.k': 3,
                          },
            },
            'l': [
                {
                    'm.n': 4,
                },
            ],
        }
        expected_result = {
            'a': {
                'b': 1,
            },
            'c': {
                'd': {
                    'e': {
                        'f': {
                            'g': 1,
                            'h': 2,
                            'i': {
                                'j': {
                                    'k': 3,
                                },
                            },
                        },
                    },
                },
            },
            'l': [
                {
                    'm': {
                        'n': 4,
                    },
                },
            ],
        }
        normalize_dotted_fields(document)
        self.assertTrue(compare_recursive(document, expected_result))


class TestOpLogBase(TestBase):
    def setUp(self):
        super(TestOpLogBase, self).setUp()
        self.test_field, self.test_value = 'ref', "1234567890123456789054321"
        self.data = {self.test_field: self.test_value}
        self.test_client = self.app.test_client()
        self.headers = [(('Content-Type', 'application/json'))]

    def oplog_reset(self):
        self.app._init_oplog()
        self.app.register_resource('oplog', self.domain['oplog'])

        settings = self.app.config['DOMAIN']['oplog']
        datasource = settings['datasource']
        schema = settings['schema']
        datasource['projection'] = {}
        self.app._set_resource_projection(datasource, schema, settings)

    def oplog_get(self, url='/oplog'):
        r = self.test_client.get(url)
        return self.parse_response(r)

    def assertOpLogEntry(self, entry, op, user=None):
        self.assertTrue('r' in entry)
        self.assertTrue('i' in entry)
        self.assertTrue(config.LAST_UPDATED in entry)
        self.assertTrue(config.DATE_CREATED in entry)
        self.assertTrue('o' in entry)
        self.assertEqual(entry['o'], op)
        self.assertTrue('127.0.0.1' in entry['ip'])
        if op in self.app.config['OPLOG_CHANGE_METHODS']:
            self.assertTrue('c' in entry)
        self.assertTrue('u' in entry)
        if user:
            self.assertTrue(user in entry['u'])
        else:
            self.assertTrue('n/a' in entry['u'])


class TestOpLogEndpointDisabled(TestOpLogBase):
    def setUp(self):
        super(TestOpLogEndpointDisabled, self).setUp()

        self.app.config['OPLOG'] = True
        from eve.default_settings import OPLOG_CHANGE_METHODS
        self.app.config['OPLOG_CHANGE_METHODS'] = OPLOG_CHANGE_METHODS
        self.oplog_reset()

    def test_post_oplog(self):
        r = self.test_client.post(self.known_resource_url,
                                  data=json.dumps(self.data),
                                  headers=self.headers,
                                  environ_base={'REMOTE_ADDR': '127.0.0.1'})

        # oplog endpoint is not available.
        r, status = self.oplog_get()
        self.assert404(status)

        # however the oplog collection has been updated.
        db = self.connection[MONGO_DBNAME]
        cursor = db.oplog.find()
        self.assertEqual(cursor.count(), 1)
        self.assertOpLogEntry(cursor[0], 'POST')


class TestOpLogEndpointEnabled(TestOpLogBase):
    def setUp(self):
        super(TestOpLogEndpointEnabled, self).setUp()

        self.app.config['OPLOG'] = True
        self.app.config['OPLOG_ENDPOINT'] = 'oplog'
        self.oplog_reset()

    def test_oplog_hook(self):
        def oplog_callback(resource, entries):
            for entry in entries:
                entry['extra'] = {'customfield': 'customvalue'}

        self.app.on_oplog_push += oplog_callback

        r = self.test_client.post(self.known_resource_url,
                                  data=json.dumps(self.data),
                                  headers=self.headers,
                                  environ_base={'REMOTE_ADDR': '127.0.0.1'})

        # oplog enpoint does not expose the 'extra' field
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'POST')
        self.assertTrue('extra' not in oplog_entry)

        # however the oplog collection has the field.
        db = self.connection[MONGO_DBNAME]
        cursor = db.oplog.find()
        self.assertEqual(cursor.count(), 1)
        oplog_entry = cursor[0]
        self.assertTrue('extra' in oplog_entry)
        self.assertTrue('customvalue' in oplog_entry['extra']['customfield'])

        # enable 'extra' field for the endpoint
        self.app.config['OPLOG_RETURN_EXTRA_FIELD'] = True
        self.oplog_reset()

        # now the oplog endpoint includes the 'extra' field
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'POST')
        self.assertTrue('extra' in oplog_entry)
        self.assertTrue('customvalue' in oplog_entry['extra']['customfield'])

    def test_post_oplog(self):
        r = self.test_client.post(self.known_resource_url,
                                  data=json.dumps(self.data),
                                  headers=self.headers,
                                  environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'POST')
        self.assertTrue('extra' not in oplog_entry)

    def test_patch_oplog(self):
        self.headers.append(('If-Match', self.item_etag))
        r = self.test_client.patch(self.item_id_url,
                                   data=json.dumps(self.data),
                                   headers=self.headers,
                                   environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'PATCH')

    def test_put_oplog(self):
        self.headers.append(('If-Match', self.item_etag))
        r = self.test_client.put(self.item_id_url,
                                 data=json.dumps(self.data),
                                 headers=self.headers,
                                 environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'PUT')

    def test_put_oplog_does_not_alter_document(self):
        """ Make sure we don't alter document ETag when performing an
        oplog_push. See #590. """
        self.headers.append(('If-Match', self.item_etag))
        r = self.test_client.put(self.item_id_url,
                                 data=json.dumps(self.data),
                                 headers=self.headers,
                                 environ_base={'REMOTE_ADDR': '127.0.0.1'})

        etag1 = json.loads(r.get_data())['_etag']
        etag2 = json.loads(
            self.test_client.get(self.item_id_url).get_data())['_etag']
        self.assertEqual(etag1, etag2)

    def test_delete_oplog(self):
        self.headers.append(('If-Match', self.item_etag))
        r = self.test_client.delete(self.item_id_url,
                                    headers=self.headers,
                                    environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'DELETE')

    def test_soft_delete_oplog(self):
        r, s = self.parse_response(self.test_client.get(self.item_id_url))
        doc_date = r[config.LAST_UPDATED]
        time.sleep(1)

        self.domain[self.known_resource]['soft_delete'] = True

        self.headers.append(('If-Match', self.item_etag))
        r = self.test_client.delete(self.item_id_url,
                                    headers=self.headers,
                                    environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'DELETE')
        self.assertTrue(doc_date != oplog_entry[config.LAST_UPDATED])

    def test_post_oplog_with_basic_auth(self):
        self.domain['contacts']['authentication'] = ValidBasicAuth
        self.headers.append(('Authorization', 'Basic YWRtaW46c2VjcmV0'))
        r = self.test_client.post(self.known_resource_url,
                                  data=json.dumps(self.data),
                                  headers=self.headers,
                                  environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'POST', 'admin')

    def test_post_oplog_with_token_auth(self):
        self.domain['contacts']['authentication'] = ValidTokenAuth
        self.headers.append(('Authorization', 'Basic dGVzdF90b2tlbjo='))
        r = self.test_client.post(self.known_resource_url,
                                  data=json.dumps(self.data),
                                  headers=self.headers,
                                  environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'POST', 'test_token')

    def test_post_oplog_with_hmac_auth(self):
        self.domain['contacts']['authentication'] = ValidHMACAuth
        self.headers.append(('Authorization', 'admin:secret'))
        r = self.test_client.post(self.known_resource_url,
                                  data=json.dumps(self.data),
                                  headers=self.headers,
                                  environ_base={'REMOTE_ADDR': '127.0.0.1'})
        r, status = self.oplog_get()
        self.assert200(status)
        self.assertEqual(len(r['_items']), 1)
        oplog_entry = r['_items'][0]
        self.assertOpLogEntry(oplog_entry, 'POST', 'admin')

    def patch(self, url, data, headers=[], content_type='application/json'):
        headers.append(('Content-Type', content_type))
        headers.append(('If-Match', self.item_etag))
        r = self.test_client.patch(url, data=json.dumps(data), headers=headers)
        return self.parse_response(r)

    def put(self, url, data, headers=[], content_type='application/json'):
        headers.append(('Content-Type', content_type))
        headers.append(('If-Match', self.item_etag))
        r = self.test_client.put(url, data=json.dumps(data), headers=headers)
        return self.parse_response(r)


class TestTickets(TestBase):
    def test_ticket_681(self):
        # See https://github.com/pyeve/eve/issues/681
        with self.app.test_request_context('not_an_existing_endpoint'):
            self.app.data.driver.db['again']



# -*- coding: utf-8 -*-

"""
    Default values in schemas
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Default values for schemas work in two steps.
    1. The schema is searched for defaults and a list of default is built.
    2. In each POST/PUT request, for each default (if any) the document is
    checked for a missing value, and if a value is missing the default is
    added.

    :copyright: (c) 2017 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""


def build_defaults(schema):
    """Build a tree of default values

    It walks the tree down looking for entries with a `default` key. In order
    to avoid empty dicts the tree will be walked up and the empty dicts will be
    removed.

    :param schema: Resource schema
    :type schema: dict
    :rtype: dict with defaults

    .. versionadded:: 0.4
    """
    # Pending schema nodes to process: loop and add defaults
    pending = set()
    # Stack of nodes to work on and clean up
    stack = [(schema, None, None, {})]
    level_schema, level_name, level_parent, current = stack[-1]
    while len(stack) > 0:
        leave = True
        if isinstance(current, list):
            level_schema = {'schema': level_schema.copy()}
        for name, value in level_schema.items():
            default_next_level = None
            if 'default' in value:
                try:
                    current[name] = value['default']
                except TypeError:
                    current.append(value['default'])
            elif value.get('type') == 'dict' and 'schema' in value:
                default_next_level = {}
            elif value.get('type') == 'list' and 'schema' in value:
                default_next_level = []

            if default_next_level is not None:
                leave = False
                next_level = add_next_level(name, current, default_next_level)
                stack.append((value['schema'], name, current, next_level))
                pending.add(id(next_level))
        pending.discard(id(current))
        if leave:
            # Leaves trigger the `walk up` till the next not processed node
            while id(current) not in pending:
                if not current and level_parent is not None:
                    try:
                        del level_parent[level_name]
                    except TypeError:
                        level_parent.remove(current)
                stack.pop()
                if len(stack) == 0:
                    break
                level_schema, level_name, level_parent, current = stack[-1]
        else:
            level_schema, level_name, level_parent, current = stack[-1]

    return current


def add_next_level(name, current, default):
    if isinstance(current, list):
        current.append(default)
    else:
        default = current.setdefault(name, default)
    return default


def resolve_default_values(document, defaults):
    """ Add any defined default value for missing document fields.

    :param document: the document being posted or replaced
    :param defaults: tree with the default values
    :type defaults: dict

    .. versionchanged:: 0.5
       Fix #417. A default value of [] for a list causes an IndexError.

    .. versionadded:: 0.2
    """
    todo = [(defaults, document)]
    while len(todo) > 0:
        defaults, document = todo.pop()
        if isinstance(defaults, list) and len(defaults):
            todo.extend((defaults[0], item) for item in document)
            continue
        for name, value in defaults.items():
            if isinstance(value, dict):
                # default dicts overwrite simple values
                existing = document.setdefault(name, {})
                if not isinstance(existing, dict):
                    document[name] = {}
                todo.append((value, document[name]))
            if isinstance(value, list) and len(value):
                existing = document.get(name)
                if not existing:
                    document.setdefault(name, value)
                    continue
                if all(isinstance(item, (dict, list)) for item in existing):
                    todo.extend((value[0], item) for item in existing)
                else:
                    document.setdefault(name, existing)
            else:
                document.setdefault(name, value)



# -*- coding: utf-8 -*-
from datetime import datetime

import json
from bson import ObjectId
from flask_pymongo import MongoClient

import eve
from eve.auth import BasicAuth
from eve.tests import TestBase
from eve.tests.test_settings import MONGO1_PASSWORD, MONGO1_USERNAME, \
    MONGO1_DBNAME, MONGO_DBNAME, \
    MONGO_HOST, MONGO_PORT


class TestMultiMongo(TestBase):
    def setUp(self):
        super(TestMultiMongo, self).setUp()

        self.setupDB2()

        schema = {
            'author': {'type': 'string'},
            'title': {'type': 'string'},
        }
        settings = {
            'schema': schema,
            'mongo_prefix': 'MONGO1'
        }

        self.app.register_resource('works', settings)

    def tearDown(self):
        super(TestMultiMongo, self).tearDown()
        self.dropDB2()

    def setupDB2(self):
        self.connection = MongoClient()
        self.connection.drop_database(MONGO1_DBNAME)
        self.connection[MONGO1_DBNAME].add_user(MONGO1_USERNAME,
                                                MONGO1_PASSWORD)
        self.bulk_insert2()

    def dropDB2(self):
        self.connection = MongoClient()
        self.connection.drop_database(MONGO1_DBNAME)
        self.connection.close()

    def bulk_insert2(self):
        _db = self.connection[MONGO1_DBNAME]
        works = self.random_works(self.known_resource_count)
        _db.works.insert(works)
        self.work = _db.works.find_one()
        self.connection.close()

    def random_works(self, num):
        works = []
        for i in range(num):
            dt = datetime.now()
            work = {
                'author': self.random_string(20),
                'title': self.random_string(30),
                eve.LAST_UPDATED: dt,
                eve.DATE_CREATED: dt,
            }
            works.append(work)
        return works


class TestMethodsAcrossMultiMongo(TestMultiMongo):
    def test_get_multidb(self):
        # test that a GET on 'works' reads from MONGO1
        id_field = self.domain['works']['id_field']
        r, s = self.get('works/%s' % self.work[id_field])
        self.assert200(s)
        self.assertEqual(r['author'], self.work['author'])

        # while 'contacts' endpoint reads from MONGO
        id_field = self.domain['contacts']['id_field']
        r, s = self.get(self.known_resource, item=self.item_id)
        self.assert200(s)
        self.assertEqual(r[id_field], self.item_id)

    def test_post_multidb(self):
        # test that a POST on 'works' stores data to MONGO1
        work = self._save_work()
        db = self.connection[MONGO1_DBNAME]
        id_field = self.domain['works']['id_field']
        new = db.works.find_one({id_field: ObjectId(work[id_field])})
        self.assertTrue(new is not None)
        self.connection.close()

        # while 'contacts' endpoint stores data to MONGO
        contact = {'ref': '1234567890123456789054321'}
        r, s = self.post(self.known_resource_url, data=contact)
        self.assert201(s)
        db = self.connection[MONGO_DBNAME]
        id_field = self.domain['contacts']['id_field']
        new = db.contacts.find_one({id_field: ObjectId(r[id_field])})
        self.assertTrue(new is not None)
        self.connection.close()

    def test_patch_multidb(self):
        # test that a PATCH on 'works' udpates data on MONGO1
        work = self._save_work()
        id_field = self.domain['works']['id_field']
        id, etag = work[id_field], work[eve.ETAG]
        changes = {'author': 'mike'}

        headers = [('Content-Type', 'application/json'), ('If-Match', etag)]
        r = self.test_client.patch('works/%s' % id, data=json.dumps(changes),
                                   headers=headers)
        self.assert200(r.status_code)

        db = self.connection[MONGO1_DBNAME]
        updated = db.works.find_one({id_field: ObjectId(id)})
        self.assertEqual(updated['author'], 'mike')
        self.connection.close()

        # while 'contacts' endpoint updates data on MONGO
        field, value = "ref", "1234567890123456789012345"
        changes = {field: value}
        headers = [('Content-Type', 'application/json'), ('If-Match',
                                                          self.item_etag)]
        id_field = self.domain['contacts']['id_field']
        r = self.test_client.patch(self.item_id_url, data=json.dumps(changes),
                                   headers=headers)
        self.assert200(r.status_code)

        db = self.connection[MONGO_DBNAME]
        updated = db.contacts.find_one({id_field: ObjectId(self.item_id)})
        self.assertEqual(updated[field], value)
        self.connection.close()

    def test_put_multidb(self):
        # test that a PUT on 'works' udpates data on MONGO1
        work = self._save_work()
        id_field = self.domain['works']['id_field']
        id, etag = work[id_field], work[eve.ETAG]
        changes = {'author': 'mike', 'title': 'Eve for dummies'}

        headers = [('Content-Type', 'application/json'), ('If-Match', etag)]
        r = self.test_client.put('works/%s' % id, data=json.dumps(changes),
                                 headers=headers)
        self.assert200(r.status_code)

        db = self.connection[MONGO1_DBNAME]
        updated = db.works.find_one({id_field: ObjectId(id)})
        self.assertEqual(updated['author'], 'mike')
        self.connection.close()

        # while 'contacts' endpoint updates data on MONGO
        field, value = "ref", "1234567890123456789012345"
        changes = {field: value}
        headers = [('Content-Type', 'application/json'), ('If-Match',
                                                          self.item_etag)]
        id_field = self.domain['contacts']['id_field']
        r = self.test_client.put(self.item_id_url, data=json.dumps(changes),
                                 headers=headers)
        self.assert200(r.status_code)

        db = self.connection[MONGO_DBNAME]
        updated = db.contacts.find_one({id_field: ObjectId(self.item_id)})
        self.assertEqual(updated[field], value)
        self.connection.close()

    def test_delete_multidb(self):
        # test that DELETE on 'works' deletes data on MONGO1
        work = self._save_work()
        id_field = self.domain['works']['id_field']
        id, etag = work[id_field], work[eve.ETAG]
        r = self.test_client.delete('works/%s' % id, headers=[('If-Match',
                                                               etag)])
        self.assert204(r.status_code)
        db = self.connection[MONGO1_DBNAME]
        lost = db.works.find_one({id_field: ObjectId(id)})
        self.assertEqual(lost, None)
        self.connection.close()

        # while 'contacts' still deletes on MONGO
        r = self.test_client.delete(self.item_id_url,
                                    headers=[('If-Match', self.item_etag)])
        self.assert204(r.status_code)
        db = self.connection[MONGO_DBNAME]
        id_field = self.domain['contacts']['id_field']
        lost = db.contacts.find_one({id_field: ObjectId(self.item_id)})
        self.assertEqual(lost, None)
        self.connection.close()

    def test_create_index_with_mongo_uri_and_prefix(self):
        self.app.config['MONGO_URI'] = 'mongodb://%s:%s/%s' % (
            MONGO_HOST, MONGO_PORT, MONGO_DBNAME)
        self.app.config['MONGO1_URI'] = 'mongodb://%s:%s/%s' % (
            MONGO_HOST, MONGO_PORT, MONGO1_DBNAME)
        settings = {
            'schema': {
                'name': {'type': 'string'},
                'other_field': {'type': 'string'},
                'lat_long': {'type': 'list'}
            },
            'mongo_indexes': {
                'name': [('name', 1)],
                'composed': [('name', 1), ('other_field', 1)],
                'arguments': ([('lat_long', "2d")], {"sparse": True})
            },
            'mongo_prefix': 'MONGO1',
        }
        self.app.register_resource('mongodb_features', settings)

        # check if index was created using MONGO1 prefix
        db = self.connection[MONGO1_DBNAME]
        self.assertTrue('mongodb_features' in db.collection_names())
        coll = db['mongodb_features']
        indexes = coll.index_information()

        # at least there is an index for the _id field plus the indexes
        self.assertTrue(len(indexes) > len(settings['mongo_indexes']))

    def _save_work(self):
        work = {'author': 'john doe', 'title': 'Eve for Dummies'}
        r, s = self.post('works', data=work)
        self.assert201(s)
        return r


class MyBasicAuth(BasicAuth):
    def check_auth(self, username, password, allowed_roles, resource, method):
        self.set_mongo_prefix('MONGO1')
        return True


class TestMultiMongoAuth(TestMultiMongo):
    def test_get_multidb(self):
        self.domain['works']['mongo_prefix'] = 'MONGO'
        self.domain['works']['public_item_methods'] = []

        headers = [('Authorization', 'Basic YWRtaW46c2VjcmV0')]

        # this will 404 since there's no 'works' collection on MONGO,
        id_field = self.domain['works']['id_field']
        r = self.test_client.get('works/%s' % self.work[id_field],
                                 headers=headers)
        self.assert404(r.status_code)

        # now set a custom auth class which sets mongo_prefix at MONGO1
        self.domain['works']['authentication'] = MyBasicAuth

        # this will 200 just fine as the custom auth class has precedence over
        # endpoint configuration.
        r = self.test_client.get('works/%s' % self.work[id_field],
                                 headers=headers)
        self.assert200(r.status_code)
        # test that we are indeed reading from the correct database instance.
        payl = json.loads(r.get_data().decode('utf-8'))
        self.assertEqual(payl['author'], self.work['author'])

        # 'contacts' still reads from MONGO
        r = self.test_client.get('%s/%s' % (self.known_resource_url,
                                            self.item_id), headers=headers)
        self.assert200(r.status_code)



# -*- coding: utf-8 -*-

SETTINGS = {
    'DEBUG': True,
    'MONGO_HOST': 'localhost',
    'MONGO_PORT': 27017,
    'MONGO_DBNAME': 'test_db',
    'DOMAIN': {'accounts': {
        'username': {
            'type': 'string',
            'minlength': 5,
            'maxlength': 20,
        },
        'password': {
            'type': 'string',
            'minlength': 5,
            'maxlength': 20,
        },
        'secret_key': {
            'type': 'string',
            'minlength': 5,
            'maxlength': 20,
        },
        'roles': {
            'type': 'string',
            'minlength': 10,
            'maxlength': 50,
        },
        'token': {
            'type': 'string',
            'minlength': 10,
            'maxlength': 50,
        },
    }}
}



import simplejson as json

from bson import ObjectId
from bson.dbref import DBRef
from eve import ETAG
from eve import ISSUES
from eve import LAST_UPDATED
from eve import STATUS
from eve import STATUS_OK
from eve.methods.put import put_internal
from eve.tests import TestBase
from eve.tests.test_settings import MONGO_DBNAME
from eve.tests.utils import DummyEvent


class TestPut(TestBase):
    # TODO consider making a base codebase out of 'patch' and 'put' tests
    def test_put_to_resource_endpoint(self):
        _, status = self.put(self.known_resource_url, data={})
        self.assert405(status)

    def test_readonly_resource(self):
        _, status = self.put(self.readonly_id_url, data={})
        self.assert405(status)

    def test_by_name(self):
        _, status = self.put(self.item_name_url, data={'key1': 'value1'})
        self.assert405(status)

    def test_ifmatch_missing(self):
        _, status = self.put(self.item_id_url, data={'key1': 'value1'})
        self.assert428(status)

    def test_ifmatch_missing_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False

    def test_ifmatch_disabled(self):
        self.app.config['IF_MATCH'] = False
        r, status = self.put(self.item_id_url,
                             data={'ref': '1234567890123456789012345'})
        self.assert200(status)
        self.assertTrue(ETAG not in r)

    def test_ifmatch_disabled_enforce_ifmatch_disabled(self):
        self.app.config['IF_MATCH'] = False
        self.app.config['ENFORCE_IF_MATCH'] = False
        r, status = self.put(
            self.item_id_url,
            data={'ref': '1234567890123456789012345'}
        )
        self.assert200(status)
        self.assertTrue(ETAG not in r)

    def test_ifmatch_bad_etag(self):
        _, status = self.put(self.item_id_url,
                             data={'key1': 'value1'},
                             headers=[('If-Match', 'not-quite-right')])
        self.assert412(status)

    def test_ifmatch_bad_etag_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False
        _, status = self.put(
            self.item_id_url,
            data={'key1': 'value1'},
            headers=[('If-Match', 'not-quite-right')]
        )
        self.assert412(status)

    def test_unique_value(self):
        r, status = self.put(self.item_id_url,
                             data={"ref": "%s" % self.alt_ref},
                             headers=[('If-Match', self.item_etag)])
        self.assertValidationErrorStatus(status)
        self.assertValidationError(r, {'ref': "value '%s' is not unique" %
                                       self.alt_ref})

    def test_allow_unknown(self):
        changes = {"unknown": "unknown"}
        r, status = self.put(self.item_id_url, data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assertValidationErrorStatus(status)
        self.assertValidationError(r, {'unknown': 'unknown field'})
        self.app.config['DOMAIN'][self.known_resource]['allow_unknown'] = True
        changes = {"unknown": "unknown", "ref": "1234567890123456789012345"}
        r, status = self.put(self.item_id_url, data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assert200(status)
        self.assertPutResponse(r, self.item_id)

    def test_put_x_www_form_urlencoded(self):
        field = "ref"
        test_value = "1234567890123456789012345"
        changes = {field: test_value}
        headers = [('If-Match', self.item_etag)]
        r, status = self.parse_response(self.test_client.put(
            self.item_id_url, data=changes, headers=headers))
        self.assert200(status)
        self.assertTrue('OK' in r[STATUS])

    def test_put_x_www_form_urlencoded_number_serialization(self):
        del(self.domain['contacts']['schema']['ref']['required'])
        field = 'anumber'
        test_value = 41
        changes = {field: test_value}
        headers = [('If-Match', self.item_etag)]
        r, status = self.parse_response(self.test_client.put(
            self.item_id_url, data=changes, headers=headers))
        self.assert200(status)
        self.assertTrue('OK' in r[STATUS])

    def test_put_referential_integrity(self):
        data = {"person": self.unknown_item_id}
        headers = [('If-Match', self.invoice_etag)]
        r, status = self.put(self.invoice_id_url, data=data, headers=headers)
        self.assertValidationErrorStatus(status)
        expected = ("value '%s' must exist in resource '%s', field '%s'" %
                    (self.unknown_item_id, 'contacts',
                     self.domain['contacts']['id_field']))
        self.assertValidationError(r, {'person': expected})

        data = {"person": self.item_id}
        r, status = self.put(self.invoice_id_url, data=data, headers=headers)
        self.assert200(status)
        self.assertPutResponse(r, self.invoice_id, 'invoices')

    def test_put_referential_integrity_list(self):
        data = {"invoicing_contacts": [self.item_id, self.unknown_item_id]}
        headers = [('If-Match', self.invoice_etag)]
        r, status = self.put(self.invoice_id_url, data=data, headers=headers)
        self.assertValidationErrorStatus(status)
        expected = ("value '%s' must exist in resource '%s', field '%s'" %
                    (self.unknown_item_id, 'contacts',
                     self.domain['contacts']['id_field']))
        self.assertValidationError(r, {'invoicing_contacts': expected})

        data = {"invoicing_contacts": [self.item_id, self.item_id]}
        r, status = self.put(self.invoice_id_url, data=data, headers=headers)
        self.assert200(status)
        self.assertPutResponse(r, self.invoice_id, 'invoices')

    def test_put_write_concern_success(self):
        # 0 and 1 are the only valid values for 'w' on our mongod instance (1
        # is the default)
        self.domain['contacts']['mongo_write_concern'] = {'w': 0}
        field = "ref"
        test_value = "X234567890123456789012345"
        changes = {field: test_value}
        _, status = self.put(self.item_id_url, data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assert200(status)

    def test_put_write_concern_fail(self):
        # should get a 500 since there's no replicaset on the mongod instance
        self.domain['contacts']['mongo_write_concern'] = {'w': 2}
        field = "ref"
        test_value = "X234567890123456789012345"
        changes = {field: test_value}
        _, status = self.put(self.item_id_url, data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assert500(status)

    def test_put_string(self):
        field = "ref"
        test_value = "1234567890123456789012345"
        changes = {field: test_value}
        r = self.perform_put(changes)
        db_value = self.compare_put_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_put_with_post_override(self):
        # POST request with PUT override turns into a PUT
        field = "ref"
        test_value = "1234567890123456789012345"
        changes = {field: test_value}
        headers = [('X-HTTP-Method-Override', 'PUT'),
                   ('If-Match', self.item_etag),
                   ('Content-Type', 'application/x-www-form-urlencoded')]
        r = self.test_client.post(self.item_id_url, data=changes,
                                  headers=headers)
        self.assert200(r.status_code)
        self.assertPutResponse(json.loads(r.get_data()), self.item_id)

    def test_put_default_value(self):
        test_field = 'title'
        test_value = "Mr."
        data = {'ref': '9234567890123456789054321'}
        r = self.perform_put(data)
        db_value = self.compare_put_with_get(test_field, r)
        self.assertEqual(test_value, db_value)

    def test_put_readonly_value_same(self):
        data = {'ref': self.item['ref'],
                'read_only_field': self.item['read_only_field']}
        r, status = self.put(self.item_id_url,
                             data=data,
                             headers=[('If-Match', self.item_etag)])
        self.assert200(status)

    def test_put_readonly_value_different(self):
        field = 'read_only_field'
        data = {'ref': self.item['ref'], field: 'somethingelse'}
        r, status = self.put(self.item_id_url,
                             data=data,
                             headers=[('If-Match', self.item_etag)])
        self.assert422(status)
        self.assertValidationError(r, {field: "field is read-only"})

    def test_put_subresource(self):
        _db = self.connection[MONGO_DBNAME]
        self.app.config['BANDWIDTH_SAVER'] = False

        # create random contact
        fake_contact = self.random_contacts(1)
        fake_contact_id = _db.contacts.insert(fake_contact)[0]

        # update first invoice to reference the new contact
        _db.invoices.update({'_id': ObjectId(self.invoice_id)},
                            {'$set': {'person': fake_contact_id}})

        # GET all invoices by new contact
        response, status = self.get('users/%s/invoices/%s' %
                                    (fake_contact_id, self.invoice_id))
        etag = response[ETAG]

        data = {"inv_number": "new_number"}
        headers = [('If-Match', etag)]
        response, status = self.put('users/%s/invoices/%s' %
                                    (fake_contact_id, self.invoice_id),
                                    data=data, headers=headers)
        self.assert200(status)
        self.assertPutResponse(response, self.invoice_id, 'peopleinvoices')
        self.assertEqual(response.get('person'), str(fake_contact_id))

    def test_put_dbref_subresource(self):
        _db = self.connection[MONGO_DBNAME]
        self.app.config['BANDWIDTH_SAVER'] = False

        # create random contact
        fake_contact = self.random_contacts(1)
        fake_contact_id = _db.contacts.insert(fake_contact)[0]

        # update first invoice to reference the new contact
        _db.invoices.update({'_id': ObjectId(self.invoice_id)},
                            {'$set': {
                                'person': fake_contact_id,
                                'persondbref':
                                    DBRef("contacts",
                                          ObjectId(fake_contact_id))}
                             })

        # GET all invoices by new contact
        response, status = self.get('users/%s/invoices/%s' %
                                    (fake_contact_id, self.invoice_id))

        self.assertEqual(response.get('persondbref')['$id'],
                         str(fake_contact_id))

        etag = response[ETAG]

        data = {"inv_number": "new_number"}
        headers = [('If-Match', etag)]
        response, status = self.put('users/%s/invoices/%s' %
                                    (fake_contact_id, self.invoice_id),
                                    data=data, headers=headers)

        self.assert200(status)
        self.assertPutResponse(response, self.invoice_id, 'peopleinvoices')

    def test_put_bandwidth_saver(self):
        changes = {'ref': '1234567890123456789012345'}

        # bandwidth_saver is on by default
        self.assertTrue(self.app.config['BANDWIDTH_SAVER'])
        r = self.perform_put(changes)
        self.assertFalse('ref' in r)
        db_value = self.compare_put_with_get(self.app.config['ETAG'], r)
        self.assertEqual(db_value, r[self.app.config['ETAG']])
        self.item_etag = r[self.app.config['ETAG']]

        # test return all fields (bandwidth_saver off)
        self.app.config['BANDWIDTH_SAVER'] = False
        r = self.perform_put(changes)
        self.assertTrue('ref' in r)
        db_value = self.compare_put_with_get(self.app.config['ETAG'], r)
        self.assertEqual(db_value, r[self.app.config['ETAG']])

    def test_put_dependency_fields_with_default(self):
        # Test that if a dependency is missing but has a default value then the
        # field is still accepted. See #353.
        del(self.domain['contacts']['schema']['ref']['required'])
        field = "dependency_field2"
        test_value = "a value"
        changes = {field: test_value}
        r = self.perform_put(changes)
        db_value = self.compare_put_with_get(field, r)
        self.assertEqual(db_value, test_value)

    def test_put_dependency_fields_with_wrong_value(self):
        # Test that if a dependency is not met, the put is refused
        del(self.domain['contacts']['schema']['ref']['required'])
        r, status = self.put(self.item_id_url,
                             data={'dependency_field3': 'value'},
                             headers=[('If-Match', self.item_etag)])
        self.assert422(status)
        r, status = self.put(self.item_id_url,
                             data={'dependency_field1': 'value',
                                   'dependency_field3': 'value'},
                             headers=[('If-Match', self.item_etag)])
        self.assert200(status)

    def test_put_custom_idfield(self):
        product = {'title': 'Awesome Hypercube'}
        r, status = self.put('products/FOOBAR', data=product)
        self.assert201(status)

    def test_put_internal(self):
        # test that put_internal is available and working properly.
        test_field = 'ref'
        test_value = "9876543210987654321098765"
        data = {test_field: test_value}
        with self.app.test_request_context(self.item_id_url):
            r, _, _, status = put_internal(
                self.known_resource, data, concurrency_check=False,
                **{'_id': self.item_id})
        db_value = self.compare_put_with_get(test_field, r)
        self.assertEqual(db_value, test_value)
        self.assert200(status)

    def test_put_internal_skip_validation(self):
        # test that put_internal is available and working properly.
        test_field = 'ref'
        test_value = "9876543210987654321098765"
        data = {test_field: test_value}
        with self.app.test_request_context(self.item_id_url):
            r, _, _, status = put_internal(
                self.known_resource, data, concurrency_check=False,
                skip_validation=True, **{'_id': self.item_id})
        db_value = self.compare_put_with_get(test_field, r)
        self.assertEqual(db_value, test_value)
        self.assert200(status)

    def test_put_etag_header(self):
        # test that Etag is always includer with response header. See #562.
        changes = {"ref": "1234567890123456789012345"}
        headers = [('Content-Type', 'application/json'),
                   ('If-Match', self.item_etag)]
        r = self.test_client.put(self.item_id_url,
                                 data=json.dumps(changes),
                                 headers=headers)
        self.assertTrue('Etag' in r.headers)

        # test that ETag is compliant to RFC 7232-2.3 and #794 is fixed.
        etag = r.headers['ETag']

        self.assertTrue(etag[0] == '"')
        self.assertTrue(etag[-1] == '"')

    def test_put_etag_header_enforce_ifmatch_disabled(self):
        self.app.config['ENFORCE_IF_MATCH'] = False
        changes = {'ref': '1234567890123456789012345'}
        headers = [('Content-Type', 'application/json'),
                   ('If-Match', self.item_etag)]
        r, status = self.put(
            self.item_id_url,
            data=json.dumps(changes),
            headers=headers
        )
        self.assertTrue(ETAG in r)
        self.assertTrue(self.item_etag != r[ETAG])

    def test_put_nested(self):
        changes = {
            'ref': '1234567890123456789012345',
            'location.city': 'a nested city',
            'location.address': 'a nested address'
        }
        r = self.perform_put(changes)
        values = self.compare_put_with_get('location', r)
        self.assertEqual(values['city'], 'a nested city')
        self.assertEqual(values['address'], 'a nested address')

    def test_put_creates_unexisting_document(self):
        id = str(ObjectId())
        url = '%s/%s' % (self.known_resource_url, id)
        id_field = self.domain[self.known_resource]['id_field']
        changes = {"ref": "1234567890123456789012345"}
        r, status = self.put(url, data=changes)
        # 201 is a creation (POST) response
        self.assert201(status)
        # new document has id_field matching the PUT endpoint
        self.assertEqual(r[id_field], str(id))

    def test_put_returns_404_on_unexisting_document(self):
        self.app.config['UPSERT_ON_PUT'] = False
        id = str(ObjectId())
        url = '%s/%s' % (self.known_resource_url, id)
        changes = {"ref": "1234567890123456789012345"}
        r, status = self.put(url, data=changes)
        self.assert404(status)

    def test_put_creates_unexisting_document_with_url_as_id(self):
        id = str(ObjectId())
        url = '%s/%s' % (self.known_resource_url, id)
        id_field = self.domain[self.known_resource]['id_field']
        changes = {"ref": "1234567890123456789012345",
                   id_field: str(ObjectId())}
        r, status = self.put(url, data=changes)
        # 201 is a creation (POST) response
        self.assert201(status)
        # new document has id_field matching the PUT endpoint
        # (eventual mismatching id_field in the payload is ignored/replaced)
        self.assertEqual(r[id_field], str(id))

    def test_put_creates_unexisting_document_fails_on_mismatching_id(self):
        id = str(ObjectId())
        id_field = self.domain[self.known_resource]['id_field']
        changes = {"ref": "1234567890123456789012345", id_field: id}
        r, status = self.put(self.item_id_url,
                             data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assert400(status)
        self.assertTrue('immutable' in r['_error']['message'])

    def test_put_type_coercion(self):
        schema = self.domain[self.known_resource]['schema']
        schema['aninteger']['coerce'] = lambda string: int(float(string))
        changes = {'ref': '1234567890123456789054321', 'aninteger': '42.3'}
        r, status = self.put(self.item_id_url, data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assert200(status)
        r, status = self.get(r['_links']['self']['href'])
        self.assertEqual(r['aninteger'], 42)

    def perform_put(self, changes):
        r, status = self.put(self.item_id_url,
                             data=changes,
                             headers=[('If-Match', self.item_etag)])
        self.assert200(status)
        self.assertPutResponse(r, self.item_id)
        return r

    def assertPutResponse(self, response, item_id, resource=None):
        id_field = self.domain[resource or self.known_resource]['id_field']
        self.assertTrue(STATUS in response)
        self.assertTrue(STATUS_OK in response[STATUS])
        self.assertFalse(ISSUES in response)
        self.assertTrue(id_field in response)
        self.assertEqual(response[id_field], item_id)
        self.assertTrue(LAST_UPDATED in response)
        self.assertTrue(ETAG in response)
        self.assertTrue('_links' in response)
        self.assertItemLink(response['_links'], item_id)

    def compare_put_with_get(self, fields, put_response):
        raw_r = self.test_client.get(self.item_id_url)
        r, status = self.parse_response(raw_r)
        self.assert200(status)
        self.assertEqual(raw_r.headers.get('ETag').replace('"', ''),
                         put_response[ETAG])
        if isinstance(fields, str):
            return r[fields]
        else:
            return [r[field] for field in fields]


class TestEvents(TestBase):
    new_ref = "0123456789012345678901234"

    def test_on_pre_PUT(self):
        devent = DummyEvent(self.before_replace)
        self.app.on_pre_PUT += devent
        self.put()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(3, len(devent.called))

    def test_on_pre_PUT_contacts(self):
        devent = DummyEvent(self.before_replace)
        self.app.on_pre_PUT_contacts += devent
        self.put()
        self.assertEqual(2, len(devent.called))

    def test_on_pre_PUT_dynamic_filter(self):
        def filter_this(resource, request, lookup):
            lookup["_id"] = self.unknown_item_id
        self.app.on_pre_PUT += filter_this
        # Would normally delete the known document; will return 404 instead.
        r, s = self.parse_response(self.put())
        self.assert201(s)

    def test_on_post_PUT(self):
        devent = DummyEvent(self.after_replace)
        self.app.on_post_PUT += devent
        self.put()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(200, devent.called[2].status_code)
        self.assertEqual(3, len(devent.called))

    def test_on_post_PUT_contacts(self):
        devent = DummyEvent(self.after_replace)
        self.app.on_post_PUT_contacts += devent
        self.put()
        self.assertEqual(200, devent.called[1].status_code)
        self.assertEqual(2, len(devent.called))

    def test_on_replace(self):
        devent = DummyEvent(self.before_replace)
        self.app.on_replace += devent
        self.put()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(self.new_ref, devent.called[1]['ref'])
        self.assertEqual(3, len(devent.called))

    def test_on_replace_contacts(self):
        devent = DummyEvent(self.before_replace)
        self.app.on_replace_contacts += devent
        self.put()
        self.assertEqual(self.new_ref, devent.called[0]['ref'])
        self.assertEqual(2, len(devent.called))

    def test_on_replaced(self):
        devent = DummyEvent(self.after_replace)
        self.app.on_replaced += devent
        self.put()
        self.assertEqual(self.known_resource, devent.called[0])
        self.assertEqual(self.new_ref, devent.called[1]['ref'])
        self.assertEqual(3, len(devent.called))

    def test_on_replaced_contacts(self):
        devent = DummyEvent(self.after_replace)
        self.app.on_replaced_contacts += devent
        self.put()
        self.assertEqual(self.new_ref, devent.called[0]['ref'])
        self.assertEqual(2, len(devent.called))

    def before_replace(self):
        db = self.connection[MONGO_DBNAME]
        contact = db.contacts.find_one(ObjectId(self.item_id))
        return contact['ref'] == self.item_name

    def after_replace(self):
        return not self.before_replace()

    def put(self):
        headers = [('Content-Type', 'application/json'),
                   ('If-Match', self.item_etag)]
        data = json.dumps({"ref": self.new_ref})
        return self.test_client.put(self.item_id_url, data=data,
                                    headers=headers)



#!/usr/bin/env python

from setuptools import setup, find_packages
DESCRIPTION = ("Python REST API for Humans.")
with open('README.rst') as f:
    LONG_DESCRIPTION = f.read()

install_requires = [
    'cerberus>=0.9.2,<0.10',
    'events>=0.2.1,<0.3',
    'simplejson>=3.3.0,<4.0',
    'werkzeug>=0.9.4,<=0.11.15',
    'markupsafe>=0.23,<1.0',
    'jinja2>=2.8,<3.0',
    'itsdangerous>=0.24,<1.0',
    'flask>=0.10.1,<=0.12',
    'pymongo>=3.4',
    'flask-pymongo>=0.4',
]

try:
    from collections import OrderedDict  # noqa
except ImportError:
    # Python 2.6 needs this back-port
    install_requires.append('ordereddict')


setup(
    name='Eve',
    version='0.7.8',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author='Nicola Iarocci',
    author_email='eve@nicolaiarocci.com',
    url='http://python-eve.org',
    license='BSD',
    platforms=["any"],
    packages=find_packages(),
    test_suite="eve.tests",
    install_requires=install_requires,
    tests_require=['redis', 'testfixtures'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-

import copy
import hashlib
from bson.json_util import dumps
from datetime import datetime, timedelta
from eve.tests import TestBase
from eve.utils import parse_request, str_to_date, config, weak_date, \
    date_to_str, querydef, document_etag, extract_key_values, \
    debug_error_message, validate_filters


class TestUtils(TestBase):
    """ collection, document and home_link methods (and resource_uri, which is
    used by all of them) are tested in 'tests.methods' since we need an active
    flaskapp context
    """

    def setUp(self):
        super(TestUtils, self).setUp()
        self.dt_fmt = config.DATE_FORMAT
        self.datestr = 'Tue, 18 Sep 2012 10:12:30 GMT'
        self.valid = datetime.strptime(self.datestr, self.dt_fmt)
        self.etag = '56eaadbbd9fa287e7270cf13a41083c94f52ab9b'

    def test_parse_request_where(self):
        self.app.config['DOMAIN'][self.known_resource]['allowed_filters'] = \
            ['ref']
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).where, None)
        with self.app.test_request_context('/?where=hello'):
            self.assertEqual(parse_request(self.known_resource).where, 'hello')

    def test_parse_request_sort(self):
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).sort, None)
        with self.app.test_request_context('/?sort=hello'):
            self.assertEqual(parse_request(self.known_resource).sort, 'hello')

    def test_parse_request_page(self):
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).page, 1)
        with self.app.test_request_context('/?page=2'):
            self.assertEqual(parse_request(self.known_resource).page, 2)
        with self.app.test_request_context('/?page=-1'):
            self.assertEqual(parse_request(self.known_resource).page, 1)
        with self.app.test_request_context('/?page=0'):
            self.assertEqual(parse_request(self.known_resource).page, 1)
        with self.app.test_request_context('/?page=1.1'):
            self.assertEqual(parse_request(self.known_resource).page, 1)
        with self.app.test_request_context('/?page=string'):
            self.assertEqual(parse_request(self.known_resource).page, 1)

    def test_parse_request_max_results(self):
        default = config.PAGINATION_DEFAULT
        limit = config.PAGINATION_LIMIT
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)
        with self.app.test_request_context('/?max_results=%d' % (limit + 1)):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             limit)
        with self.app.test_request_context('/?max_results=2'):
            self.assertEqual(parse_request(self.known_resource).max_results, 2)
        with self.app.test_request_context('/?max_results=-1'):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)
        with self.app.test_request_context('/?max_results=0'):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)
        with self.app.test_request_context('/?max_results=1.1'):
            self.assertEqual(parse_request(self.known_resource).max_results, 1)
        with self.app.test_request_context('/?max_results=string'):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)

    def test_parse_request_max_results_disabled_pagination(self):
        self.app.config['DOMAIN'][self.known_resource]['pagination'] = False
        default = 0
        limit = config.PAGINATION_LIMIT
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)
        with self.app.test_request_context('/?max_results=%d' % (limit + 1)):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             limit + 1)
        with self.app.test_request_context('/?max_results=2'):
            self.assertEqual(parse_request(self.known_resource).max_results, 2)
        with self.app.test_request_context('/?max_results=-1'):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)
        with self.app.test_request_context('/?max_results=0'):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)
        with self.app.test_request_context('/?max_results=1.1'):
            self.assertEqual(parse_request(self.known_resource).max_results, 1)
        with self.app.test_request_context('/?max_results=string'):
            self.assertEqual(parse_request(self.known_resource).max_results,
                             default)

    def test_parse_request_if_modified_since(self):
        ims = 'If-Modified-Since'
        with self.app.test_request_context():
            self.assertEqual(parse_request(
                self.known_resource).if_modified_since, None)
        with self.app.test_request_context(headers=None):
            self.assertEqual(
                parse_request(self.known_resource).if_modified_since, None)
        with self.app.test_request_context(headers={ims: self.datestr}):
            self.assertEqual(
                parse_request(self.known_resource).if_modified_since,
                self.valid + timedelta(seconds=1))
        with self.app.test_request_context(headers={ims: 'not-a-date'}):
            self.assertRaises(ValueError, parse_request, self.known_resource)
        with self.app.test_request_context(
            headers={ims:
                     self.datestr.replace('GMT', 'UTC')}):
            self.assertRaises(ValueError, parse_request, self.known_resource)
            self.assertRaises(ValueError, parse_request, self.known_resource)

    def test_parse_request_if_none_match(self):
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).if_none_match,
                             None)
        with self.app.test_request_context(headers=None):
            self.assertEqual(parse_request(self.known_resource).if_none_match,
                             None)
        with self.app.test_request_context(headers={'If-None-Match':
                                                    self.etag}):
            self.assertEqual(parse_request(self.known_resource).if_none_match,
                             self.etag)

    def test_parse_request_if_match(self):
        with self.app.test_request_context():
            self.assertEqual(parse_request(self.known_resource).if_match, None)
        with self.app.test_request_context(headers=None):
            self.assertEqual(parse_request(self.known_resource).if_match, None)
        with self.app.test_request_context(headers={'If-Match': self.etag}):
            self.assertEqual(parse_request(self.known_resource).if_match,
                             self.etag)

    def test_weak_date(self):
        with self.app.test_request_context():
            self.app.config['DATE_FORMAT'] = '%Y-%m-%d'
            self.assertEqual(weak_date(self.datestr), self.valid +
                             timedelta(seconds=1))

    def test_str_to_date(self):
        self.assertEqual(str_to_date(self.datestr), self.valid)
        self.assertRaises(ValueError, str_to_date, 'not-a-date')
        self.assertRaises(ValueError, str_to_date,
                          self.datestr.replace('GMT', 'UTC'))

    def test_date_to_str(self):
        self.assertEqual(date_to_str(self.valid), self.datestr)

    def test_querydef(self):
        self.assertEqual(querydef(max_results=10), '?max_results=10')
        self.assertEqual(querydef(page=10), '?page=10')
        self.assertEqual(querydef(where='wherepart'), '?where=wherepart')
        self.assertEqual(querydef(sort='sortpart'), '?sort=sortpart')

        self.assertEqual(querydef(where='wherepart', sort='sortpart'),
                         '?where=wherepart&sort=sortpart')
        self.assertEqual(querydef(max_results=10, sort='sortpart'),
                         '?max_results=10&sort=sortpart')

    def test_document_etag(self):
        test = {'key1': 'value1', 'another': 'value2'}
        challenge = dumps(test, sort_keys=True).encode('utf-8')
        with self.app.test_request_context():
            self.assertEqual(hashlib.sha1(challenge).hexdigest(),
                             document_etag(test))

    def test_document_etag_ignore_fields(self):
        test = {'key1': 'value1', 'key2': 'value2'}
        ignore_fields = ["key2"]
        test_without_ignore = {'key1': 'value1'}
        challenge = dumps(test_without_ignore, sort_keys=True).encode('utf-8')
        with self.app.test_request_context():
            self.assertEqual(hashlib.sha1(challenge).hexdigest(),
                             document_etag(test, ignore_fields))

        # not required fields can not be present
        test = {'key1': 'value1', 'key2': 'value2'}
        ignore_fields = ["key3"]
        test_without_ignore = {'key1': 'value1', 'key2': 'value2'}
        challenge = dumps(test_without_ignore, sort_keys=True).encode('utf-8')
        with self.app.test_request_context():
            self.assertEqual(hashlib.sha1(challenge).hexdigest(),
                             document_etag(test, ignore_fields))

        # ignore fiels nested using doting notation
        test = {'key1': 'value1', 'dict': {'key2': 'value2', 'key3': 'value3'}}
        ignore_fields = ['dict.key2']
        test_without_ignore = {'key1': 'value1', 'dict': {'key3': 'value3'}}
        challenge = dumps(test_without_ignore, sort_keys=True).encode('utf-8')
        with self.app.test_request_context():
            self.assertEqual(hashlib.sha1(challenge).hexdigest(),
                             document_etag(test, ignore_fields))

    def test_extract_key_values(self):
        test = {
            'key1': 'value1',
            'key2': {
                'key1': 'value2',
                'nested': {
                    'key1': 'value3'
                }
            }
        }
        self.assertEqual(list(extract_key_values('key1', test)),
                         ['value1', 'value2', 'value3'])

    def test_debug_error_message(self):
        with self.app.test_request_context():
            self.app.config['DEBUG'] = False
            self.assertEqual(debug_error_message('An error message'), None)
            self.app.config['DEBUG'] = True
            self.assertEqual(debug_error_message('An error message'),
                             'An error message')

    def test_validate_filters(self):
        self.app.config['DOMAIN'][self.known_resource]['allowed_filters'] = []
        with self.app.test_request_context():
            self.assertTrue('key' in validate_filters(
                {'key': 'val'},
                self.known_resource))
            self.assertTrue('key' in validate_filters(
                {'key': ['val1', 'val2']},
                self.known_resource))
            self.assertTrue('key' in validate_filters(
                {'key': {'$in': ['val1', 'val2']}},
                self.known_resource))
            self.assertTrue('key' in validate_filters(
                {'$or': [{'key': 'val1'}, {'key': 'val2'}]},
                self.known_resource))
            self.assertTrue('$or' in validate_filters(
                {'$or': 'val'},
                self.known_resource))
            self.assertTrue('$or' in validate_filters(
                {'$or': {'key': 'val1'}},
                self.known_resource))
            self.assertTrue('$or' in validate_filters(
                {'$or': ['val']},
                self.known_resource))

        self.app.config['DOMAIN'][self.known_resource]['allowed_filters'] = \
            ['key']
        with self.app.test_request_context():
            self.assertTrue(validate_filters(
                {'key': 'val'},
                self.known_resource) is None)
            self.assertTrue(validate_filters(
                {'key': ['val1', 'val2']},
                self.known_resource) is None)
            self.assertTrue(validate_filters(
                {'key': {'$in': ['val1', 'val2']}},
                self.known_resource) is None)
            self.assertTrue(validate_filters(
                {'$or': [{'key': 'val1'}, {'key': 'val2'}]},
                self.known_resource) is None)


class DummyEvent(object):
    """
    Even handler that records the call parameters and asserts a check

    Usage::

        app = Eve()
        app.on_my_event = DummyEvent(element_not_deleted)

    In the test::

        assert app.on_my_event.called[0] == expected_param_0
    """
    def __init__(self, check, deepcopy=False):
        """
        :param check: method checking the state of something during the event.
        :type: check: callable returning bool
        :param deepcopy: Do we need to store a copy of the argument calls? In
            some events arguments are changed after the event, so keeping a
            reference to the original object doesn't allow a test to check what
            was passed. The default is False.
        :type deepcopy: bool
        """
        self.__called = None
        self.__check = check
        self.__deepcopy = deepcopy

    def __call__(self, *args):
        assert self.__check()
        # In some method the arguments are changed after the events
        if self.__deepcopy:
            args = copy.deepcopy(args)
        self.__called = args

    @property
    def called(self):
        """
        The results of the call to the event.

        :rtype: It returns None if the event hasn't been called or a tuple with
            the positional arguments of the last call if called.
        """
        return self.__called
